@@ -24,11 +24,12 @@
 #include <config.h>
 extern "C" {
   #include <glib/gi18n.h>
+  #include <glib/gfileutils.h> // for g_build_filename
 }
 #include <pan/general/debug.h>
 #include <pan/general/file-util.h>
-#include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/general/time-elapsed.h>
 #include "data-impl.h"
 
@@ -55,7 +56,9 @@ DataImpl :: DataImpl (bool unit_test, int cache_megs, DataIO * io):
   _cache (get_cache_path(), cache_megs),
   _unit_test (unit_test),
   _data_io (io),
-  _descriptions_loaded (false)
+  _descriptions_loaded (false),
+  newsrc_autosave_id (0),
+  newsrc_autosave_timeout (0)
 {
   rebuild_backend ();
 }
