@@ -40,6 +40,7 @@
 #include <pan/data-impl/data-io.h>
 #include <pan/data-impl/article-filter.h>
 #include <pan/data-impl/profiles.h>
+#include <pan/data-impl/memchunk.h>
 
 namespace pan
 {
@@ -198,7 +199,7 @@ namespace pan
          */
         struct Server {
           Numbers _read;
-          unsigned long _xover_high;
+          uint64_t _xover_high;
           Server(): _xover_high(0) {}
         };
         typedef Loki::AssocVector<Quark,Server> servers_t;
@@ -357,8 +358,8 @@ namespace pan
         int _ref;
         bool _dirty;
         nodes_t _nodes;
-        std::deque<Article> _art_chunk;
-        std::deque<ArticleNode> _node_chunk;
+        MemChunk<Article> _art_chunk;
+        MemChunk<ArticleNode> _node_chunk;
 
         GroupHeaders();
         ~GroupHeaders ();
@@ -442,7 +443,7 @@ namespace pan
           const Quark _group;
           DataImpl & _data;
           nodes_t _nodes;
-          std::deque<ArticleNode> _node_chunk;
+          MemChunk<ArticleNode> _node_chunk;
           FilterInfo _filter;
           Data::ShowType _show_type;
           struct NodeMidCompare;
@@ -500,7 +501,7 @@ namespace pan
                                                size_t                end_line,
                                                bool                  do_rescore);
 
-      void rescore_articles (const Quark& group, const quarks_t mids);
+      virtual void rescore_articles (const Quark& group, const quarks_t mids);
 
       virtual void rescore ();
 
@@ -586,16 +587,16 @@ namespace pan
 
       virtual void xover_unref   (const Quark           & group);
 
-      virtual unsigned long get_xover_high (const Quark & group,
-                                            const Quark & server) const;
+      virtual uint64_t get_xover_high (const Quark & group,
+                                       const Quark & server) const;
 
       virtual void set_xover_high (const Quark          & group,
                                    const Quark          & server,
-                                   const unsigned long    high);
+                                   const uint64_t         high);
 
        virtual void set_xover_low (const Quark          & group,
                                    const Quark          & server,
-                                   const unsigned long    low);
+                                   const uint64_t         low);
 
 
     /**
@@ -613,6 +614,19 @@ namespace pan
     public:
 
       const ArticleFilter _article_filter;
+
+    private:
+      guint newsrc_autosave_id;
+      guint newsrc_autosave_timeout;
+    public:
+      bool in_newsrc_cb;
+      void set_newsrc_autosave_timeout(guint seconds)
+        {newsrc_autosave_timeout = seconds;}
+      void save_newsrc_files()
+      {
+        save_newsrc_files(*_data_io);
+        newsrc_autosave_id = 0;
+      }
   };
 }
 
