@@ -29,10 +29,6 @@ extern "C" {
 #include "pad.h"
 #include "gui.h"
 
-#if !GTK_CHECK_VERSION(2,6,0)
-#define GTK_STOCK_EDIT GTK_STOCK_OPEN
-#endif
-
 using pan::PanUI;
 
 namespace
@@ -89,7 +85,7 @@ namespace
 
       //std::cerr << "registered icon " << my_builtin_icons[i].name << std::endl;
 
-      gdk_pixbuf_unref (pixbuf);
+      g_object_unref (pixbuf);
       gtk_icon_set_unref (icon_set);
     }
 
@@ -110,6 +106,7 @@ namespace
   void do_unsubscribe_selected_groups  (GtkAction*) { pan_ui->do_unsubscribe_selected_groups(); }
   void do_save_articles                (GtkAction*) { pan_ui->do_save_articles(); }
   void do_save_articles_from_nzb       (GtkAction*) { pan_ui->do_save_articles_from_nzb(); }
+  void do_save_articles_to_nzb	       (GtkAction*) { pan_ui->do_save_articles_to_nzb(); }
   void do_print                        (GtkAction*) { pan_ui->do_print(); }
   void do_import_tasks                 (GtkAction*) { pan_ui->do_import_tasks(); }
   void do_cancel_latest_task           (GtkAction*) { pan_ui->do_cancel_latest_task(); }
@@ -324,10 +321,15 @@ namespace
       G_CALLBACK(do_save_articles) },
 
     { "save-articles-from-nzb", GTK_STOCK_SAVE,
-      N_("_Save Articles from this NZB..."), NULL,
+      N_("Save Articles from this _NZB..."), "<control><shift>S",
       N_("Save Articles from this NZB"),
       G_CALLBACK(do_save_articles_from_nzb) },
 
+    { "save-articles-to-nzb", GTK_STOCK_SAVE,
+      N_("Save Articles to an NZB _File..."), "<control><shift>F",
+      N_("Save Articles to an NZB File"),
+      G_CALLBACK(do_save_articles_to_nzb) },
+
     { "print", GTK_STOCK_PRINT,
       NULL, "<control>P",
       NULL,
@@ -596,17 +598,10 @@ namespace
       NULL,
       G_CALLBACK(do_tip_jar) },
 
-#if GTK_CHECK_VERSION(2,6,0)
     { "about-pan", GTK_STOCK_ABOUT,
       N_("_About"), NULL, 
       NULL,
       G_CALLBACK(do_about_pan) }
-#else
-    { "about-pan", NULL,
-      N_("_About"), NULL, 
-      NULL,
-      G_CALLBACK(do_about_pan) }
-#endif
   };
 
   void prefs_toggle_callback_impl (GtkToggleAction * action)
@@ -628,6 +623,7 @@ namespace
     { "size-pictures-to-fit",     NULL, N_("Size Pictures to _Fit"),          NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "monospace-font-enabled",   NULL, N_("Use _Monospace Font"),             "C", NULL, G_CALLBACK(prefs_toggle_callback_impl), false },
     { "focus-on-image",           NULL, N_("Set Focus to Images"),            NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
+    { "highlight-urls",           NULL, N_("Highlight _URLs"),                 NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
 
 
     { "work-online", "ICON_ONLINE", N_("_Work Online"), "L", NULL, G_CALLBACK(do_work_online), true },
