@@ -24,12 +24,14 @@
 extern "C" {
   #include <glib/gi18n.h>
   #include <gtk/gtk.h>
+  #include <gdk/gdk.h>
   #include <gdk/gdkkeysyms.h>
   #include <gmime/gmime.h>
 }
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/general/utf8-utils.h>
 #include <pan/usenet-utils/gnksa.h>
 #include <pan/usenet-utils/mime-utils.h>
@@ -40,6 +42,7 @@ extern "C" {
 #include "tango-colors.h"
 #include "xface.h"
 #include "url.h"
+#include "gtk_compat.h"
 
 #define FIRST_PICTURE "first-picture"
 
@@ -158,7 +161,7 @@ namespace
   void free_cursors (void)
   {
     for (int i=0; i<CURSOR_QTY; ++i)
-      gdk_cursor_destroy (cursors[i]);
+      gdk_cursor_unref (cursors[i]);
   }
 
   void ensure_cursors_created (GtkWidget * w)
@@ -170,8 +173,9 @@ namespace
       GdkDisplay * display (gtk_widget_get_display (w));
 
       int width, height;
-      GtkStyle * style (gtk_widget_get_style (w));
       const GtkIconSize size (GTK_ICON_SIZE_LARGE_TOOLBAR);
+#if !GTK_CHECK_VERSION(3,0,0)
+      GtkStyle * style (gtk_widget_get_style (w));
       const GtkTextDirection dir (GTK_TEXT_DIR_NONE);
       const GtkStateType state (GTK_STATE_PRELIGHT);
 
@@ -186,6 +190,17 @@ namespace
       g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, NULL);
       cursors[CURSOR_ZOOM_OUT] = gdk_cursor_new_from_pixbuf (display, pixbuf, width/2, height/2);
       g_object_unref (G_OBJECT(pixbuf));
+#else
+      GdkPixbuf * pixbuf = gtk_widget_render_icon_pixbuf (w, GTK_STOCK_ZOOM_IN, size);
+      g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, NULL);
+      cursors[CURSOR_ZOOM_IN] = gdk_cursor_new_from_pixbuf (display, pixbuf, width/2, height/2);
+      g_object_unref (G_OBJECT(pixbuf));
+
+      pixbuf = gtk_widget_render_icon_pixbuf (w, GTK_STOCK_ZOOM_OUT, size);
+      g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, NULL);
+      cursors[CURSOR_ZOOM_OUT] = gdk_cursor_new_from_pixbuf (display, pixbuf, width/2, height/2);
+      g_object_unref (G_OBJECT(pixbuf));
+#endif
 
       cursors[CURSOR_IBEAM] = gdk_cursor_new (GDK_XTERM);
       cursors[CURSOR_HREF] = gdk_cursor_new (GDK_HAND2);
@@ -401,7 +416,9 @@ BodyPane :: mouse_button_pressed (GtkWidget *w, GdkEventButton *event)
         const int offset (gtk_text_iter_get_offset (&iter));
         GtkTextBuffer * buf (gtk_text_view_get_buffer (GTK_TEXT_VIEW(w)));
         const bool fullsize (toggle_fullsize_flag (buf));
-        resize_picture_at_iter (buf, &iter, fullsize, &w->allocation, pix_tag);
+        GtkAllocation aloc;
+        gtk_widget_get_allocation(w, &aloc);
+        resize_picture_at_iter (buf, &iter, fullsize, &aloc, pix_tag);
         gtk_text_iter_set_offset (&iter, offset);
         set_cursor_from_iter (event->window, w, &iter);
 
@@ -481,8 +498,8 @@ namespace
     gboolean handled (false);
 
     g_return_val_if_fail (GTK_IS_TEXT_VIEW(w), false);
-    const bool up = event->keyval==GDK_Up || event->keyval==GDK_KP_Up;
-    const bool down = event->keyval==GDK_Down || event->keyval==GDK_KP_Down;
+    const bool up = event->keyval==GDK_KEY_Up || event->keyval==GDK_KEY_KP_Up;
+    const bool down = event->keyval==GDK_KEY_Down || event->keyval==GDK_KEY_KP_Down;
 
     if (up || down)
     {
@@ -490,13 +507,13 @@ namespace
       gtk_text_view_place_cursor_onscreen (GTK_TEXT_VIEW(w));
       GtkAdjustment * adj = gtk_scrolled_window_get_vadjustment (
                                                       GTK_SCROLLED_WINDOW(scroll));
-      gdouble val = adj->value;
+      gdouble val = gtk_adjustment_get_value(adj);
       if (up)
-        val -= adj->step_increment;
+        val -= gtk_adjustment_get_step_increment(adj);
       else
-        val += adj->step_increment;
-      val = MAX(val, adj->lower);
-      val = MIN(val, adj->upper-adj->page_size);
+        val += gtk_adjustment_get_step_increment(adj);
+      val = MAX(val, gtk_adjustment_get_lower(adj) );
+      val = MIN(val, gtk_adjustment_get_upper(adj) - gtk_adjustment_get_page_size(adj) );
       gtk_adjustment_set_value (adj, val);
     }
 
@@ -661,7 +678,8 @@ namespace
                              const StringView    & body_in,
                              bool                  mute_quotes,
                              bool                  show_smilies,
-                             bool                  do_markup)
+                             bool                  do_markup,
+                             bool                  do_urls)
   {
     g_return_if_fail (buffer!=0);
     g_return_if_fail (GTK_IS_TEXT_BUFFER(buffer));
@@ -782,11 +800,13 @@ namespace
     }
     
     // colorize urls
-    StringView area;
-    StringView march (v_all);
-    while ((url_find (march, area))) {
-      set_section_tag (buffer, &start, v_all, area, "url", REPLACE);
-      march = march.substr (area.str + area.len, 0);
+    if (do_urls) {
+      StringView area;
+      StringView march (v_all);
+      while ((url_find (march, area))) {
+        set_section_tag (buffer, &start, v_all, area, "url", REPLACE);
+        march = march.substr (area.str + area.len, 0);
+      }
     }
 
     // do this last, since it alters the text instead of just marking it up
@@ -803,7 +823,7 @@ namespace
    * Generates a GtkPixmap object from a given GMimePart that contains an image.
    * Used for displaying attached pictures inline.
    */
-  GdkPixbuf* get_pixbuf_from_gmime_part (const GMimePart * part)
+  GdkPixbuf* get_pixbuf_from_gmime_part (GMimePart * part)
   {
     GdkPixbufLoader * l (gdk_pixbuf_loader_new ());
     GError * err (0);
@@ -815,12 +835,37 @@ namespace
       GMimeStream * mem_stream (g_mime_stream_mem_new ());
       g_mime_data_wrapper_write_to_stream (wrapper, mem_stream);
       GByteArray * buffer (GMIME_STREAM_MEM(mem_stream)->buffer);
-      if (buffer->len)
-          gdk_pixbuf_loader_write (l, (guchar*)buffer->data, buffer->len, &err);
+      gsize bytesLeft = buffer->len;
+      guchar * data = buffer->data;
+
+      // ticket #467446 - workaround gdkpixbuf <= 2.12.x's 
+      // jpg loader bug (#494667) by feeding the loader in
+      // smaller chunks
+      while( bytesLeft > 0 )
+      {
+          const gsize n = MIN( 4096, bytesLeft );
+
+          gdk_pixbuf_loader_write (l, data, n, &err);
+          if (err) {
+            Log::add_err (err->message);
+            g_clear_error (&err);
+            break;
+          }
+
+          bytesLeft -= n;
+          data += n;
+      }
+
       g_object_unref (mem_stream);
-      g_object_unref (wrapper);
     }
-  
+
+    // flush the loader
+    gdk_pixbuf_loader_close (l, &err);
+    if (err) {
+      Log::add_err (err->message);
+      g_clear_error (&err);
+    }
+
     // create the pixbuf
     GdkPixbuf * pixbuf (0);
     if (!err)
@@ -831,11 +876,6 @@ namespace
     }
 
     // cleanup
-    gdk_pixbuf_loader_close (l, &err);
-    if (err) {
-      Log::add_err (err->message);
-      g_clear_error (&err);
-    }
     if (pixbuf)
       g_object_ref (G_OBJECT(pixbuf));
     g_object_unref (G_OBJECT(l));
@@ -853,7 +893,7 @@ BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
     return;
 
   GMimePart * part = GMIME_PART (obj);
-  const GMimeContentType * type = g_mime_object_get_content_type (GMIME_OBJECT (part));
+  GMimeContentType * type = g_mime_object_get_content_type (GMIME_OBJECT (part));
 
   // decide whether or not this part is a picture
   bool is_image (g_mime_content_type_is_type (type, "image", "*"));
@@ -910,15 +950,18 @@ BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
   else if (g_mime_content_type_is_type (type, "text", "*"))
   {
     const char * fallback_charset (_charset.c_str());
+    const char * p_flowed (g_mime_object_get_content_type_parameter(obj,"format"));
+    const bool flowed (g_strcmp0 (p_flowed, "flowed") == 0);
     std::string str = mime_part_to_utf8 (part, fallback_charset);
 
     if (!str.empty() && _prefs.get_flag ("wrap-article-body", false))
-      str = _tm.fill (str);
+      str = _tm.fill (str, flowed);
 
     const bool do_mute (_prefs.get_flag ("mute-quoted-text", false));
     const bool do_smilies (_prefs.get_flag ("show-smilies-as-graphics", true));
     const bool do_markup (_prefs.get_flag ("show-text-markup", true));
-    append_text_buffer_nolock (&_tm, _buffer, str, do_mute, do_smilies, do_markup);
+    const bool do_urls (_prefs.get_flag ("highlight-urls", true));
+    append_text_buffer_nolock (&_tm, _buffer, str, do_mute, do_smilies, do_markup, do_urls);
     is_done = true;
   }
 
@@ -935,15 +978,13 @@ BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
   }
 }
 void
-BodyPane :: foreach_part_cb (GMimeObject* o, gpointer self)
+BodyPane :: foreach_part_cb (GMimeObject* /*parent*/, GMimeObject* o, gpointer self)
 {
-  if (GMIME_IS_MULTIPART (o))
-    g_mime_multipart_foreach (GMIME_MULTIPART (o), foreach_part_cb, self);
-  else {
-    BodyPane * pane = static_cast<BodyPane*>(self);
-    GtkWidget * w (pane->_text);
-    pane->append_part (o, &w->allocation);
-  }
+  BodyPane * pane = static_cast<BodyPane*>(self);
+  GtkWidget * w (pane->_text);
+  GtkAllocation aloc;
+  gtk_widget_get_allocation(w, &aloc);
+  pane->append_part (o, &aloc);
 }
 
 
@@ -957,7 +998,7 @@ namespace
                               const char    * key,
                               const char    * fallback_charset)
   {
-    const char * val (message ? g_mime_message_get_header (message, key) : "");
+    const char * val (message ? g_mime_object_get_header ((GMimeObject *)message, key) : "");
     const std::string utf8_val (header_to_utf8 (val, fallback_charset));
     char * e (0);
     if (strcmp (key, "From"))
@@ -994,7 +1035,7 @@ namespace
                           const char    * key,
                           const char    * fallback_charset)
   {
-    const char * val (msg ? g_mime_message_get_header (msg, key) : "");
+    const char * val (msg ? g_mime_object_get_header ((GMimeObject *) msg, key) : "");
     return add_header_line (s, key_i18n, key, val, fallback_charset);
   }
 }
@@ -1016,19 +1057,19 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
 
   // conditional headers...
   if (message) {
-    const StringView newsgroups (g_mime_message_get_header (message, "Newsgroups"));
+    const StringView newsgroups (g_mime_object_get_header ((GMimeObject *) message, "Newsgroups"));
     if (newsgroups.strchr(',')) {
       l = add_header_line (s, message, _("Newsgroups"), "Newsgroups", fallback_charset);
       w = std::max (w, l);
     }
-    const StringView followup_to (g_mime_message_get_header (message, "Followup-To"));
+    const StringView followup_to (g_mime_object_get_header ((GMimeObject *) message, "Followup-To"));
     if (!followup_to.empty() && (followup_to!=newsgroups)) {
       l = add_header_line (s, message, _("Followup-To"), "Followup-To", fallback_charset);
       w = std::max (w, l);
     }
-    const StringView reply_to (g_mime_message_get_header (message, "Reply-To"));
+    const StringView reply_to (g_mime_object_get_header ((GMimeObject *) message, "Reply-To"));
     if (!reply_to.empty()) {
-      const StringView from (g_mime_message_get_header (message, "From"));
+      const StringView from (g_mime_object_get_header ((GMimeObject *) message, "From"));
       StringView f_addr, f_name, rt_addr, rt_name;
       GNKSA :: do_check_from (from, f_addr, f_name, false);
       GNKSA :: do_check_from (reply_to, rt_addr, rt_name, false);
@@ -1042,19 +1083,35 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
   s.resize (s.size()-1); // remove trailing linefeed
   gtk_label_set_markup (GTK_LABEL(_headers), s.c_str());
 
-#if GTK_CHECK_VERSION(2,6,0)
   // ellipsize mode is useless w/o this in expander...
   gtk_label_set_width_chars (GTK_LABEL(_headers), (int)w);
-#endif
 
   // set the x-face...
-  GdkPixbuf * pixbuf (0);
-  const char * pch = message ? g_mime_message_get_header (message, "X-Face") : 0;
-  if (pch && _xface->window)
-    pixbuf = pan_gdk_pixbuf_create_from_x_face (gtk_widget_get_colormap(_xface), _xface->window, pch);
-  gtk_image_set_from_pixbuf (GTK_IMAGE(_xface), pixbuf);
-  if (pixbuf)
+  gtk_image_clear(GTK_IMAGE(_xface));
+  const char * pch = message ? g_mime_object_get_header ((GMimeObject *) message, "X-Face") : 0;
+  if (pch && gtk_widget_get_window(_xface) )
+  {
+    GdkPixbuf *pixbuf = NULL;
+    pixbuf = pan_gdk_pixbuf_create_from_x_face (pch);
+    gtk_image_set_from_pixbuf (GTK_IMAGE(_xface), pixbuf);
     g_object_unref (pixbuf);
+  }
+  // set the face
+  gtk_image_clear(GTK_IMAGE(_face));
+  pch = message ? g_mime_object_get_header ((GMimeObject *) message, "Face") : 0;
+  if (pch && gtk_widget_get_window(_face))
+  {
+    GMimeEncoding dec;
+    g_mime_encoding_init_decode(&dec, GMIME_CONTENT_ENCODING_BASE64);
+    guchar buf[1024];
+    int len = g_mime_encoding_step(&dec, pch, strlen(pch), (char*)buf);
+    GdkPixbufLoader *pl = gdk_pixbuf_loader_new_with_type( "png", NULL);
+    gdk_pixbuf_loader_write(pl, buf, len, NULL);
+    gdk_pixbuf_loader_close(pl, NULL);
+    GdkPixbuf *pixbuf = gdk_pixbuf_loader_get_pixbuf(pl);
+    gtk_image_set_from_pixbuf (GTK_IMAGE(_face), pixbuf);
+    g_object_unref(pl);
+  }
 
   // set the terse headers...
   s.clear ();
@@ -1064,10 +1121,8 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
   s += _(" at ");
   add_bold_header_value (s, message, "Date", fallback_charset);
   gtk_label_set_markup (GTK_LABEL(_terse), s.c_str());
-#if GTK_CHECK_VERSION(2,6,0)
   // ellipsize mode is useless w/o this in expander...
   gtk_label_set_width_chars (GTK_LABEL(_terse), (int)s.size());
-#endif
 
   // clear the text buffer...
   GtkTextIter start, end;
@@ -1080,7 +1135,7 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
   // maybe add the headers
   const bool do_show_headers (_prefs.get_flag ("show-all-headers", false));
   if (message && do_show_headers) {
-    char * headers (g_mime_message_get_headers (message));
+    char * headers (g_mime_object_get_headers ((GMimeObject *) message));
     GtkTextIter end;
     gtk_text_buffer_get_end_iter (_buffer, &end);
     StringView line, v(headers);
@@ -1098,7 +1153,7 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
 
   // set the text buffer...
   if (message)
-    g_mime_message_foreach_part (message, foreach_part_cb, this);
+    g_mime_message_foreach (message, foreach_part_cb, this);
 
   // if there was a picture, scroll to it.
   // otherwise scroll to the top of the body.
@@ -1195,9 +1250,9 @@ BodyPane :: refresh_scroll_visible_state ()
 {
   GtkScrolledWindow * w (GTK_SCROLLED_WINDOW (_scroll));
   GtkAdjustment * adj = gtk_scrolled_window_get_hadjustment (w);
-  _hscroll_visible = adj->page_size < adj->upper;
+  _hscroll_visible = gtk_adjustment_get_page_size(adj) < gtk_adjustment_get_upper(adj);
   adj = gtk_scrolled_window_get_vadjustment (w);
-  _vscroll_visible = adj->page_size < adj->upper;
+  _vscroll_visible = gtk_adjustment_get_page_size(adj) < gtk_adjustment_get_upper(adj);
 }
 
 // show*_cb exist to ensure that _hscroll_visible and _vscroll_visible
@@ -1210,7 +1265,7 @@ BodyPane :: show_idle_cb (gpointer pane)
   return false;
 }
 void
-BodyPane :: show_cb (GtkWidget* w, gpointer pane)
+BodyPane :: show_cb (GtkWidget* w G_GNUC_UNUSED, gpointer pane)
 {
   g_idle_add (show_idle_cb, pane);
 }
@@ -1248,9 +1303,13 @@ BodyPane :: text_size_allocated_idle ()
 
   // walk through the buffer looking for pictures to resize
   GtkTextTag * tag (get_named_tag_from_view (_text, "pixbuf"));
+  GtkAllocation aloc;
   for (;;) {
     if (gtk_text_iter_begins_tag (&iter, tag))
-      resize_picture_at_iter (buf, &iter, fullsize, &_text->allocation, tag);
+    {
+      gtk_widget_get_allocation(_text, &aloc);
+      resize_picture_at_iter (buf, &iter, fullsize, &aloc, tag);
+    }
     if (!gtk_text_iter_forward_char (&iter))
       break;
     if (!gtk_text_iter_forward_to_tag_toggle (&iter, tag))
@@ -1259,8 +1318,8 @@ BodyPane :: text_size_allocated_idle ()
 }
 
 void
-BodyPane :: text_size_allocated (GtkWidget     * text,
-                                 GtkAllocation * allocation,
+BodyPane :: text_size_allocated (GtkWidget     * text        G_GNUC_UNUSED,
+                                 GtkAllocation * allocation  G_GNUC_UNUSED,
                                  gpointer        pane)
 {
   if (!text_size_allocated_idle_tag)
@@ -1272,7 +1331,7 @@ BodyPane :: text_size_allocated (GtkWidget     * text,
 ***/
 
 void
-BodyPane :: copy_url_cb (GtkMenuItem *mi, gpointer pane)
+BodyPane :: copy_url_cb (GtkMenuItem *mi G_GNUC_UNUSED, gpointer pane)
 {
   static_cast<BodyPane*>(pane)->copy_url ();
 }
@@ -1290,7 +1349,7 @@ BodyPane :: populate_popup_cb (GtkTextView *v, GtkMenu *m, gpointer pane)
   static_cast<BodyPane*>(pane)->populate_popup(v, m);
 }
 void
-BodyPane :: populate_popup (GtkTextView *v, GtkMenu *m)
+BodyPane :: populate_popup (GtkTextView *v G_GNUC_UNUSED, GtkMenu *m)
 {
   // menu separator comes first.
   GtkWidget * mi = gtk_menu_item_new();
@@ -1338,30 +1397,27 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   gtk_box_pack_start (GTK_BOX(vbox), w, false, false, 0);
 
   _terse = gtk_label_new ("Expander");
-  g_object_ref (_terse);
-  gtk_object_sink (GTK_OBJECT(_terse));
+  g_object_ref_sink (G_OBJECT(_terse));
   gtk_misc_set_alignment (GTK_MISC(_terse), 0.0f, 0.5f);
   gtk_label_set_use_markup (GTK_LABEL(_terse), true);
-  //gtk_label_set_selectable (GTK_LABEL(_terse), true);
-#if GTK_CHECK_VERSION(2,6,0)
+  gtk_label_set_selectable (GTK_LABEL(_terse), true);
   gtk_label_set_ellipsize (GTK_LABEL(_terse), PANGO_ELLIPSIZE_MIDDLE);
-#endif
   gtk_widget_show (_terse);
 
   GtkWidget * hbox = _verbose = gtk_hbox_new (false, 0);
-  g_object_ref (_verbose);
-  gtk_object_sink (GTK_OBJECT(_verbose));
+  g_object_ref_sink (G_OBJECT(_verbose));
   w = _headers = gtk_label_new ("Headers");
-  //gtk_label_set_selectable (GTK_LABEL(_headers), TRUE);
+  gtk_label_set_selectable (GTK_LABEL(_headers), TRUE);
   gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
-#if GTK_CHECK_VERSION(2,6,0)
   gtk_label_set_ellipsize (GTK_LABEL(w), PANGO_ELLIPSIZE_MIDDLE);
-#endif
   gtk_label_set_use_markup (GTK_LABEL(w), true);
   gtk_box_pack_start (GTK_BOX(hbox), w, true, true, PAD_SMALL);
   w = _xface = gtk_image_new ();
   gtk_widget_set_size_request (w, 48, 48);
   gtk_box_pack_start (GTK_BOX(hbox), w, false, false, PAD_SMALL);
+  w = _face = gtk_image_new ();
+  gtk_widget_set_size_request (w, 48, 48);
+  gtk_box_pack_start (GTK_BOX(hbox), w, false, false, PAD_SMALL);
   gtk_widget_show_all (_verbose);
 
   // setup
@@ -1439,18 +1495,22 @@ BodyPane :: read_more_or_less (bool more)
   GtkAdjustment * v = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(parent));
 
   // figure out how far we scroll
+  const double v_value = gtk_adjustment_get_value(v);
+  const double v_upper = gtk_adjustment_get_upper(v);
+  const double v_lower = gtk_adjustment_get_lower(v);
+  const double v_page_size = gtk_adjustment_get_page_size(v);
   const int arbitrary_font_height_pixels_hack (18);
-  const float inc (v->page_size - arbitrary_font_height_pixels_hack);
-  const gfloat val (CLAMP (v->value + (more ? inc : -inc),
-                           v->lower,
-                           MAX(v->upper,v->page_size)-MIN(v->upper,v->page_size)));
+  const float inc (v_page_size - arbitrary_font_height_pixels_hack);
+  const gfloat val (CLAMP (v_value + (more ? inc : -inc),
+                           v_lower,
+                           MAX(v_upper,v_page_size)-MIN(v_upper,v_page_size)));
 
   // if we can scroll, do so.
   bool handled (false);
-  if (v->upper>=v->page_size && val!=v->value)
+  if (v_upper>=v_page_size && val!=v_value)
   {
     if (_prefs.get_flag ("smooth-scrolling", true))
-      sylpheed_textview_smooth_scroll_do (v, v->value, val, smooth_scrolling_speed);
+      sylpheed_textview_smooth_scroll_do (v, v_value, val, smooth_scrolling_speed);
     else
       gtk_adjustment_set_value (v, val);
 
@@ -1493,7 +1553,7 @@ namespace
                           const char   * fallback_charset_1,
                           const char   * fallback_charset_2)
   {
-    const StringView v (g_mime_message_get_header (msg, key));
+    const StringView v (g_mime_object_get_header ((GMimeObject *) msg, key));
     std::string s;
     if (!v.empty())
       s = header_to_utf8 (v, fallback_charset_1, fallback_charset_2);
@@ -1506,24 +1566,17 @@ namespace
     std::string body;
   };
 
-  void get_utf8_body_foreach_part (GMimeObject *o, gpointer user_data)
+  void get_utf8_body_foreach_part (GMimeObject* /*parent*/, GMimeObject *o,
+                                   gpointer user_data)
   {
-    if (GMIME_IS_MULTIPART(o))
+    GMimePart * part;
+    GMimeContentType * type = g_mime_object_get_content_type (o);
+    const bool is_text (g_mime_content_type_is_type (type, "text", "*"));
+    if (is_text)
     {
-      g_mime_multipart_foreach (GMIME_MULTIPART(o),
-                                get_utf8_body_foreach_part,
-                                user_data);
-    }
-    else
-    {
-      GMimePart * part = GMIME_PART (o);
-      const GMimeContentType * type = g_mime_object_get_content_type (o);
-      const bool is_text (g_mime_content_type_is_type (type, "text", "*"));
-      if (is_text)
-      {
-        ForeachPartData *data (static_cast<ForeachPartData*>(user_data));
-        data->body += mime_part_to_utf8 (part, data->fallback_charset.c_str());
-      }
+      part = GMIME_PART (o);
+      ForeachPartData *data (static_cast<ForeachPartData*>(user_data));
+      data->body += mime_part_to_utf8 (part, data->fallback_charset.c_str());
     }
   }
 
@@ -1534,7 +1587,7 @@ namespace
     if (fallback_charset)
       tmp.fallback_charset = fallback_charset;
     if (source)
-      g_mime_message_foreach_part (source, get_utf8_body_foreach_part, &tmp);
+      g_mime_message_foreach (source, get_utf8_body_foreach_part, &tmp);
     return tmp.body;
   }
 }
@@ -1547,10 +1600,12 @@ BodyPane :: create_followup_or_reply (bool is_reply)
   if (_message)
   {
     msg = g_mime_message_new (false);
+    GMimeObject *msg_obj = (GMimeObject*)msg;
+    GMimeObject *_message_obj = (GMimeObject*)_message;
 
     // fallback character encodings
     const char * group_charset (_charset.c_str());
-    const GMimeContentType * type (g_mime_object_get_content_type (GMIME_OBJECT(_message)));
+    GMimeContentType * type (g_mime_object_get_content_type (GMIME_OBJECT(_message)));
     const char * message_charset (type ? g_mime_content_type_get_parameter (type, "charset") : 0);
 
     ///
@@ -1564,14 +1619,14 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     const std::string reply_to   (get_header (_message, "Reply-To",    message_charset, group_charset));
     if (is_reply || fup_to=="poster") {
       const std::string& to (reply_to.empty() ? from : reply_to);
-      g_mime_message_add_recipients_from_string (msg, GMIME_RECIPIENT_TYPE_TO, to.c_str());
+      pan_g_mime_message_add_recipients_from_string (msg, GMIME_RECIPIENT_TYPE_TO, to.c_str());
     } else {
       const std::string& groups (fup_to.empty() ? newsgroups : fup_to);
-      g_mime_message_add_header (msg, "Newsgroups", groups.c_str());
+      g_mime_object_append_header ((GMimeObject *) msg, "Newsgroups", groups.c_str());
     }
 
     // Subject:
-    StringView v = g_mime_message_get_header (_message, "Subject");
+    StringView v = g_mime_object_get_header (_message_obj, "Subject");
     std::string h = header_to_utf8 (v, message_charset, group_charset);
     std::string val (normalize_subject_re (h));
     if (val.find ("Re:") != 0) // add "Re: " if we don't have one
@@ -1580,22 +1635,22 @@ BodyPane :: create_followup_or_reply (bool is_reply)
 
     // attribution lines
 
-    const char * cpch = g_mime_message_get_header (_message, "From");
+    const char * cpch = g_mime_object_get_header (_message_obj, "From");
     h = header_to_utf8 (cpch, message_charset, group_charset);
-    g_mime_message_add_header (msg, "X-Draft-Attribution-Author", h.c_str());
+    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Author", h.c_str());
 
     cpch = g_mime_message_get_message_id (_message);
     h = header_to_utf8 (cpch, message_charset, group_charset);
-    g_mime_message_add_header (msg, "X-Draft-Attribution-Id", h.c_str());
+    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Id", h.c_str());
 
-    char * tmp = g_mime_message_get_date_string (_message);
+    char * tmp = g_mime_message_get_date_as_string (_message);
     h = header_to_utf8 (tmp, message_charset, group_charset);
-    g_mime_message_add_header (msg, "X-Draft-Attribution-Date", h.c_str());
+    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Date", h.c_str());
     g_free (tmp);
 
     // references
     const char * header = "References";
-    v = g_mime_message_get_header (_message, header);
+    v = g_mime_object_get_header (_message_obj, header);
     val.assign (v.str, v.len);
     if (!val.empty())
       val += ' ';
@@ -1603,7 +1658,7 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     val += g_mime_message_get_message_id (_message);
     val += ">";
     val = GNKSA :: trim_references (val);
-    g_mime_message_add_header (msg, header, val.c_str());
+    g_mime_object_append_header (msg_obj, header, val.c_str());
 
     ///
     ///  BODY
@@ -1639,17 +1694,18 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     // set the clone's content object with our modified body
     GMimeStream * stream = g_mime_stream_mem_new ();
     g_mime_stream_write_string (stream, s.c_str());
-    GMimeDataWrapper * wrapper = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_8BIT);
+    GMimeDataWrapper * wrapper = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_8BIT);
     GMimePart * part = g_mime_part_new ();
     GMimeContentType * new_type = g_mime_content_type_new_from_string ("text/plain; charset=UTF-8");
-    g_mime_part_set_content_type (part, new_type);
+    g_mime_object_set_content_type ((GMimeObject *) part, new_type);
     g_mime_part_set_content_object (part, wrapper);
-    g_mime_part_set_encoding (part, GMIME_PART_ENCODING_8BIT);
+    g_mime_part_set_content_encoding (part, GMIME_CONTENT_ENCODING_8BIT);
     g_mime_message_set_mime_part (msg, GMIME_OBJECT(part));
+    g_object_unref (new_type);
     g_object_unref (wrapper);
     g_object_unref (part);
     g_object_unref (stream);
-//std::cerr << LINE_ID << " here is the modified clone\n [" << g_mime_message_to_string(msg) << ']' << std::endl;
+//std::cerr << LINE_ID << " here is the modified clone\n [" << g_mime_object_to_string((GMimeObject *)msg) << ']' << std::endl;
   }
 
   return msg;
@@ -1666,31 +1722,32 @@ BodyPane :: refresh_fonts ()
   const bool monospace_font_enabled = _prefs.get_flag ("monospace-font-enabled", false);
 
   if (!body_pane_font_enabled && !monospace_font_enabled)
-    gtk_widget_modify_font (_text, 0);
+    gtk_widget_override_font (_text, 0);
   else {
     const std::string str (monospace_font_enabled
       ? _prefs.get_string ("monospace-font", "Monospace 10")
       : _prefs.get_string ("body-pane-font", "Sans 10"));
     PangoFontDescription * pfd (pango_font_description_from_string (str.c_str()));
-    gtk_widget_modify_font (_text, pfd);
+    gtk_widget_override_font (_text, pfd);
     pango_font_description_free (pfd);
   }
 }
 
 void
-BodyPane :: on_prefs_flag_changed (const StringView& key, bool value)
+BodyPane :: on_prefs_flag_changed (const StringView& key, bool value G_GNUC_UNUSED)
 {
   if ((key=="body-pane-font-enabled") || (key=="monospace-font-enabled"))
     refresh_fonts ();
 
   if ((key=="wrap-article-body") || (key=="mute-quoted-text") ||
       (key=="show-smilies-as-graphics") || (key=="show-all-headers") ||
-      (key=="size-pictures-to-fit") || (key=="show-text-markup"))
+      (key=="size-pictures-to-fit") || (key=="show-text-markup") ||
+      (key=="highlight-urls") )
     refresh ();
 }
 
 void
-BodyPane :: on_prefs_string_changed (const StringView& key, const StringView& value)
+BodyPane :: on_prefs_string_changed (const StringView& key, const StringView& value G_GNUC_UNUSED)
 {
   if ((key=="body-pane-font") || (key=="monospace-font"))
     refresh_fonts ();
@@ -1698,7 +1755,7 @@ BodyPane :: on_prefs_string_changed (const StringView& key, const StringView& va
 }
 
 void
-BodyPane :: on_prefs_color_changed (const StringView& key, const GdkColor& color)
+BodyPane :: on_prefs_color_changed (const StringView& key, const GdkColor& color G_GNUC_UNUSED)
 {
   if (key.strstr ("body-pane-color") != 0)
     refresh_colors ();
