@@ -25,15 +25,15 @@ extern "C" {
   #include <gtk/gtk.h>
 }
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/general/quark.h>
 #include <pan/general/text-match.h>
 #include <pan/general/time-elapsed.h>
 #include <pan/data/data.h>
 #include "group-pane.h"
 #include "pad.h"
-#include "sexy-icon-entry.h"
+#include "gtk_compat.h"
 
 using namespace pan;
 
@@ -146,7 +146,7 @@ GroupPane :: get_full_selection () const
 }
 
 void
-GroupPane ::  do_popup_menu (GtkWidget *treeview, GdkEventButton *event, gpointer pane_g)
+GroupPane ::  do_popup_menu (GtkWidget*, GdkEventButton *event, gpointer pane_g)
 {
   GroupPane * self (static_cast<GroupPane*>(pane_g));
   GtkWidget * menu (self->_action_manager.get_action_widget ("/group-pane-popup"));
@@ -166,9 +166,9 @@ namespace
 }
 
 void
-GroupPane :: on_row_activated (GtkTreeView          * treeview,
-                               GtkTreePath          * path,
-                               GtkTreeViewColumn    * col,
+GroupPane :: on_row_activated (GtkTreeView          * ,
+                               GtkTreePath          * ,
+                               GtkTreeViewColumn    * ,
                                gpointer               pane_g)
 {
   // user may have activated by selecting the row... wait for the GtkTreeSelection to catch up
@@ -179,7 +179,7 @@ namespace
 {
   bool row_collapsed_or_expanded (false);
 
-  void row_collapsed_or_expanded_cb (GtkTreeView *view, GtkTreeIter *iter, GtkTreePath *path, gpointer unused)
+  void row_collapsed_or_expanded_cb (GtkTreeView*, GtkTreeIter*, GtkTreePath*, gpointer)
   {
     row_collapsed_or_expanded = true;
   }
@@ -302,7 +302,7 @@ namespace
   }
 
   void
-  delete_rows (gpointer data, GObject * dead_object)
+  delete_rows (gpointer data, GObject*)
   {
     delete [] static_cast<MyRow*>(data);
   }
@@ -312,7 +312,7 @@ namespace
   struct NoopRowDispose: public PanTreeStore::RowDispose
   {
     virtual ~NoopRowDispose () { }
-    virtual void row_dispose (PanTreeStore *store, PanTreeStore::Row* row) { }
+    virtual void row_dispose (PanTreeStore*, PanTreeStore::Row*) { }
   };
 
   NoopRowDispose noop_row_dispose;
@@ -473,7 +473,7 @@ GroupPane :: on_group_read (const Quark& groupname)
        _dirty_groups_idle_tag = g_timeout_add (333, dirty_groups_idle, this);
 }
 void
-GroupPane :: on_group_counts (const Quark& groupname, unsigned long unread, unsigned long total)
+GroupPane :: on_group_counts (const Quark& groupname, unsigned long, unsigned long)
 {
   on_group_read (groupname);
 }
@@ -494,10 +494,21 @@ namespace
       gtk_tree_path_free (path);
     }
   }
+  const char * mode_strings [] =
+  {
+    N_("Group"),
+    N_("Group (regex)")
+  };
+
+  enum
+  {
+    GROUP,
+    GROUP_REGEX
+  };
 }
 
 void
-GroupPane :: set_filter (const std::string& search_text)
+GroupPane :: set_filter (const std::string& search_text, int mode)
 {
   GtkTreeView * view = GTK_TREE_VIEW(_tree_view);
 
@@ -509,7 +520,10 @@ GroupPane :: set_filter (const std::string& search_text)
   TextMatch match;
   TextMatch * pmatch (0);
   if (!search_text.empty()) {
-    match.set (search_text, TextMatch::CONTAINS, false);
+    if (mode == GROUP)
+      match.set (search_text, TextMatch::CONTAINS, false);
+    else if (mode == GROUP_REGEX)
+      match.set (search_text, TextMatch::REGEX, false);
     pmatch = &match;
   }
 
@@ -540,6 +554,7 @@ namespace
   guint activate_soon_tag (0u);
 
   std::string search_text;
+  int mode;
 
  void set_search_entry (GtkWidget * entry, const char * s)
   {
@@ -549,29 +564,39 @@ namespace
   }
 
   gboolean search_entry_focus_in_cb (GtkWidget     * w,
-                                     GdkEventFocus * unused1,
-                                     gpointer        unused2)
+                                     GdkEventFocus * ,
+                                     gpointer        )
   {
+#if !GTK_CHECK_VERSION(3,0,0)
     gtk_widget_modify_text (w, GTK_STATE_NORMAL, NULL); // resets
+#else
+    gtk_widget_override_color(w, GTK_STATE_FLAG_NORMAL, NULL);
+#endif
     set_search_entry (w, search_text.c_str());
     return false;
   }
 
   void refresh_search_entry (GtkWidget * w)
   {
-    if (search_text.empty() && !GTK_WIDGET_HAS_FOCUS(w))
+    if (search_text.empty() && !gtk_widget_has_focus(w))
     {
+#if !GTK_CHECK_VERSION(3,0,0)
       GdkColor c;
       c.pixel = 0;
-      c.red = c.green = c.blue = 0xAAAA; // light grey
+      c.red = c.green = c.blue = 0xAAAA;
       gtk_widget_modify_text (w, GTK_STATE_NORMAL, &c);
-      set_search_entry (w, _("Group Name"));
+#else
+      GdkRGBA c;
+      gdk_rgba_parse (&c, "0xAAA");
+      gtk_widget_override_color(w, GTK_STATE_FLAG_NORMAL, &c);
+#endif
+      set_search_entry (w, _(mode_strings[mode]));
     }
   }
 
   gboolean search_entry_focus_out_cb (GtkWidget     * w,
-                                      GdkEventFocus * unused1,
-                                      gpointer        unused2)
+                                      GdkEventFocus * ,
+                                      gpointer        )
   {
     refresh_search_entry (w);
     return false;
@@ -579,7 +604,7 @@ namespace
 
   void search_activate (GroupPane * pane)
   {
-    pane->set_filter (search_text);
+    pane->set_filter (search_text, mode);
   }
 
   void remove_activate_soon_tag ()
@@ -591,18 +616,27 @@ namespace
     }
   }
 
-  void search_entry_activated (GtkEntry * entry, gpointer pane_gpointer)
+  void search_entry_activated (GtkEntry*, gpointer pane_gpointer)
   {
     search_activate (static_cast<GroupPane*>(pane_gpointer));
     remove_activate_soon_tag ();
   }
 
-  void clear_button_clicked_cb (SexyIconEntry *e, SexyIconEntryPosition icon_pos, int button, gpointer pane_gpointer)
+  void entry_icon_released (GtkEntry*, GtkEntryIconPosition icon_pos, GdkEventButton *, gpointer menu)
   {
-    set_search_entry (GTK_WIDGET(e), "");
-    refresh_search_entry (GTK_WIDGET(e));
-    search_text.clear ();
-    search_entry_activated (NULL, pane_gpointer);
+    if (icon_pos == GTK_ENTRY_ICON_PRIMARY)
+      gtk_menu_popup (GTK_MENU(menu), 0, 0, 0, 0, 0, gtk_get_current_event_time());
+  }
+
+  void clear_button_clicked_cb (GtkEntry * e, GtkEntryIconPosition icon_pos, GdkEventButton *, gpointer pane_gpointer)
+  {
+    if (icon_pos == GTK_ENTRY_ICON_SECONDARY)
+    {
+      set_search_entry (GTK_WIDGET(e), "");
+      refresh_search_entry (GTK_WIDGET(e));
+      search_text.clear ();
+      search_entry_activated (NULL, pane_gpointer);
+    }
   }
 
   gboolean activated_timeout_cb (gpointer pane_gpointer)
@@ -623,6 +657,21 @@ namespace
     search_text = gtk_entry_get_text (GTK_ENTRY(e));
     bump_activate_soon_tag (static_cast<GroupPane*>(pane_gpointer));
   }
+
+  // when the search mode is changed via the menu,
+  // update our state variable and bump the activate timeout.
+  void search_menu_toggled_cb (GtkCheckMenuItem  * menu_item,
+                               gpointer            entry_g)
+  {
+    if (gtk_check_menu_item_get_active  (menu_item))
+    {
+      mode = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(menu_item), "MODE"));
+      refresh_search_entry (GTK_WIDGET(entry_g));
+      GroupPane * h = (GroupPane*) g_object_get_data (
+                                             G_OBJECT(entry_g), "group-pane");
+      bump_activate_soon_tag (h);
+    }
+  }
 }
 
 void
@@ -637,11 +686,11 @@ namespace
   bool shorten;
 
   void
-  render_group_name (GtkTreeViewColumn * col,
+  render_group_name (GtkTreeViewColumn * ,
                      GtkCellRenderer   * renderer,
                      GtkTreeModel      * model,
                      GtkTreeIter       * iter,
-                     gpointer            user_data)
+                     gpointer            )
   {
     PanTreeStore * tree (PAN_TREE_STORE(model));
     MyRow * row (dynamic_cast<MyRow*>(tree->get_row (iter)));
@@ -657,7 +706,7 @@ namespace
       g_snprintf (buf, sizeof(buf), " (%lu)", unread);
       group_name += buf;
     }
-    //if (unread || total) {
+    //if (unread || total)
       //if (unread)
       //  g_snprintf (buf, sizeof(buf), _(" (%lu of %lu)"), unread, total);
       //else
@@ -778,11 +827,11 @@ GroupPane :: read_next_group ()
 namespace
 {
   gboolean
-  select_func (GtkTreeSelection * selection,
+  select_func (GtkTreeSelection * ,
                GtkTreeModel     * model,
                GtkTreePath      * path,
-               gboolean           path_currently_selected,
-               gpointer           unused)
+               gboolean           ,
+               gpointer           )
   {
     GtkTreeIter iter;
     PanTreeStore * store (PAN_TREE_STORE(model));
@@ -796,25 +845,48 @@ namespace
 GtkWidget*
 GroupPane :: create_filter_entry ()
 {
-  GtkWidget * entry = sexy_icon_entry_new ();
-
-  GtkWidget * icon = gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
-  sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, GTK_IMAGE(icon));
-  sexy_icon_entry_set_icon_highlight(SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, true);
-  gtk_widget_set_size_request (entry, 133, -1);
-
+  GtkWidget * entry = gtk_entry_new ();
+  g_object_set_data (G_OBJECT(entry), "group-pane", this);
+  gtk_entry_set_icon_from_stock( GTK_ENTRY( entry ),
+                                 GTK_ENTRY_ICON_SECONDARY,
+                                 GTK_STOCK_CLEAR );
+  gtk_entry_set_icon_from_stock( GTK_ENTRY( entry ),
+                                 GTK_ENTRY_ICON_PRIMARY,
+                                 GTK_STOCK_FIND );
+//  gtk_widget_set_size_request (entry, 133, -1);
   _action_manager.disable_accelerators_when_focused (entry);
   g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), NULL);
   g_signal_connect (entry, "focus-out-event", G_CALLBACK(search_entry_focus_out_cb), NULL);
   g_signal_connect (entry, "activate", G_CALLBACK(search_entry_activated), this);
-  g_signal_connect (entry, "icon_released", G_CALLBACK(clear_button_clicked_cb), this);
+  g_signal_connect (entry, "icon-release", G_CALLBACK(clear_button_clicked_cb), this);
   entry_changed_tag = g_signal_connect (entry, "changed", G_CALLBACK(search_entry_changed_by_user), this);
+
+  GtkWidget * menu = gtk_menu_new ();
+  bool regex = _prefs.get_flag ("use-regex", false);
+  if ( regex == true )
+    mode = 1;
+  else
+    mode = 0;
+  GSList * l = 0;
+  for (int i=0, qty=G_N_ELEMENTS(mode_strings); i<qty; ++i)
+  {
+    GtkWidget * w = gtk_radio_menu_item_new_with_label (l, _(mode_strings[i]));
+    l = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM(w));
+    g_object_set_data (G_OBJECT(w), "MODE", GINT_TO_POINTER(i));
+    g_signal_connect (w, "toggled", G_CALLBACK(search_menu_toggled_cb),entry);
+    if (mode == i)
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(w), TRUE);
+    gtk_menu_shell_append (GTK_MENU_SHELL(menu), w);
+    gtk_widget_show (w);
+  }
+  g_signal_connect (entry, "icon-release", G_CALLBACK(entry_icon_released), menu);
+
   refresh_search_entry (entry);
   return entry;
 }
 
 void
-GroupPane :: on_selection_changed (GtkTreeSelection * sel, gpointer pane_gpointer)
+GroupPane :: on_selection_changed (GtkTreeSelection*, gpointer pane_gpointer)
 {
   GroupPane * self (static_cast<GroupPane*>(pane_gpointer));
   Quark group (self->get_first_selection());
@@ -847,12 +919,8 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
   g_object_unref (G_OBJECT(_tree_store)); // will die with the view
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW(_tree_view), false);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(_tree_view), false);
-#if GTK_CHECK_VERSION(2,8,0)
   gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW(_tree_view), true);
-#endif
-#if GTK_CHECK_VERSION(2,10,0)
   gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW(_tree_view), true);
-#endif
   expand_iterators (iters, GTK_TREE_MODEL(_tree_store), GTK_TREE_VIEW(_tree_view));
 
   GtkTreeSelection * selection (gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view)));
@@ -912,24 +980,24 @@ void
 GroupPane :: refresh_font ()
 {
   if (!_prefs.get_flag ("group-pane-font-enabled", false))
-    gtk_widget_modify_font (_tree_view, 0);
+    gtk_widget_override_font (_tree_view, 0);
   else {
     const std::string str (_prefs.get_string ("group-pane-font", "Sans 10"));
     PangoFontDescription * pfd (pango_font_description_from_string (str.c_str()));
-    gtk_widget_modify_font (_tree_view, pfd);
+    gtk_widget_override_font (_tree_view, pfd);
     pango_font_description_free (pfd);
   }
 }
 
 void
-GroupPane :: on_prefs_flag_changed (const StringView& key, bool value)
+GroupPane :: on_prefs_flag_changed (const StringView& key, bool)
 {
   if (key == "group-pane-font-enabled")
     refresh_font ();
 }
 
 void
-GroupPane :: on_prefs_string_changed (const StringView& key, const StringView& value)
+GroupPane :: on_prefs_string_changed (const StringView& key, const StringView&)
 {
   if (key == "group-pane-font")
     refresh_font ();
