@@ -25,11 +25,12 @@ extern "C" {
 #include <pan/general/debug.h>
 #include <pan/general/macros.h>
 #include <pan/data/data.h>
-#include "e-charset-picker.h"
+#include "e-charset-combo-box.h"
 #include "group-prefs-dialog.h"
 #include "hig.h"
 #include "pad.h"
 #include "pan-file-entry.h"
+#include "gtk_compat.h"
 
 using namespace pan;
 
@@ -39,33 +40,19 @@ namespace
   {
     delete static_cast<GroupPrefsDialog*>(castme);
   }
-
-#if !GTK_CHECK_VERSION(2,6,0)
-  char* gtk_combo_box_get_active_text (GtkComboBox * combo_box)
-  {
-    char * text (0);
-    GtkTreeIter  iter;
-    if (gtk_combo_box_get_active_iter (combo_box, &iter)) {
-      GtkTreeModel * model = gtk_combo_box_get_model (combo_box);
-      gtk_tree_model_get (model, &iter, 0, &text, -1);
-    }
-    return text;
-  }
-#endif
 }
-
+  
 void
 GroupPrefsDialog :: save_from_gui ()
 {
   // charset...
-  char * tmp = e_charset_picker_get_charset (_charset);
+  const char * tmp = e_charset_combo_box_get_charset (E_CHARSET_COMBO_BOX(_charset));
   _group_prefs.set_string (_group, "character-encoding", tmp);
-  g_free (tmp);
 
   // posting profile...
   std::string profile;
-  if (GTK_WIDGET_SENSITIVE (_profile)) {
-    char * pch (gtk_combo_box_get_active_text (GTK_COMBO_BOX(_profile)));
+  if (gtk_widget_get_sensitive (_profile)) {
+    char * pch (gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT(_profile)));
     _group_prefs.set_string (_group, "posting-profile", pch);
     g_free (pch);
   }
@@ -91,16 +78,16 @@ namespace
                              const Quark      & group,
                              const GroupPrefs & group_prefs)
   {
-    GtkWidget * w = gtk_combo_box_new_text ();
-    GtkComboBox * combo (GTK_COMBO_BOX (w));
+    GtkWidget * w = gtk_combo_box_text_new ();
+    GtkComboBoxText * combo (GTK_COMBO_BOX_TEXT (w));
 
     typedef std::set<std::string> unique_strings_t;
     const unique_strings_t profiles (data.get_profile_names ());
 
     if (profiles.empty())
     {
-      gtk_combo_box_append_text (combo, _("No Profiles defined in Edit|Posting Profiles."));
-      gtk_combo_box_set_active (combo, 0);
+      gtk_combo_box_text_append_text (combo, _("No Profiles defined in Edit|Posting Profiles."));
+      gtk_combo_box_set_active (GTK_COMBO_BOX(combo), 0);
       gtk_widget_set_sensitive (w, false);
     }
     else
@@ -111,9 +98,9 @@ namespace
         if (*it == s)
           sel_index = i;
         ++i;
-        gtk_combo_box_append_text (combo, it->c_str());
+        gtk_combo_box_text_append_text (combo, it->c_str());
       }
-      gtk_combo_box_set_active (combo, sel_index);
+      gtk_combo_box_set_active (GTK_COMBO_BOX(combo), sel_index);
     }
 
     return w;
@@ -144,9 +131,8 @@ GroupPrefsDialog :: GroupPrefsDialog (Data         & data,
   g_snprintf (buf, sizeof(buf), _("Properties for %s"), group.c_str());
   HIG::workarea_add_section_title (t, &row, buf);
     HIG :: workarea_add_section_spacer (t, row, 3);
-    _charset = e_charset_picker_new (_group_prefs.get_string (group, "character-encoding", "UTF-8").c_str());
-    w = gtk_option_menu_new ();
-    gtk_option_menu_set_menu (GTK_OPTION_MENU (w), _charset);
+    _charset = w = e_charset_combo_box_new( );
+    e_charset_combo_box_set_charset( E_CHARSET_COMBO_BOX(_charset), _group_prefs.get_string (group, "character-encoding", "UTF-8").c_str());
     HIG :: workarea_add_row (t, &row, _("Character _encoding:"), w);
     w = _save_path = file_entry_new (_("Directory for Saving Attachments"));
     char * pch = g_build_filename (g_get_home_dir(), "News", NULL);
@@ -156,9 +142,9 @@ GroupPrefsDialog :: GroupPrefsDialog (Data         & data,
     HIG :: workarea_add_row (t, &row, _("Directory for _saving attachments:"), w);
     w = _profile = create_profiles_combo_box (data, group, group_prefs);
     l = HIG :: workarea_add_row (t, &row, _("Posting _profile:"), w);
-    gtk_widget_set_sensitive (l, GTK_WIDGET_SENSITIVE(w));
+    gtk_widget_set_sensitive (l, gtk_widget_get_sensitive(w));
 
   gtk_widget_show_all (t);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), t, true, true, 0);
+  gtk_box_pack_start ( GTK_BOX( gtk_dialog_get_content_area( GTK_DIALOG( dialog))), t, true, true, 0);
   _root = dialog;
 }
