@@ -21,6 +21,7 @@
 #include <map>
 #include <string>
 #include <sstream>
+#include <fstream>
 extern "C" {
   #include <sys/types.h> // for chmod
   #include <sys/stat.h> // for chmod
@@ -31,6 +32,7 @@ extern "C" {
 #include <pan/general/file-util.h>
 #include <pan/general/macros.h>
 #include <pan/usenet-utils/scorefile.h>
+#include <pan/usenet-utils/mime-utils.h>
 #include <pan/tasks/task-article.h>
 #include <pan/tasks/task-groups.h>
 #include <pan/tasks/task-xover.h>
@@ -38,8 +40,8 @@ extern "C" {
 #include <pan/icons/pan-pixbufs.h>
 #include "actions.h"
 #include "body-pane.h"
-#include "e-charset-picker.h"
 #include "dl-headers-ui.h"
+#include "e-charset-dialog.h"
 #include "group-pane.h"
 #include "group-prefs-dialog.h"
 #include "header-pane.h"
@@ -60,18 +62,14 @@ extern "C" {
 #include "server-ui.h"
 #include "task-pane.h"
 #include "url.h"
+#include "gtk_compat.h"
 
 namespace pan
 {
   void
-  g_object_ref_sink_pan (GObject * o)
+  pan_box_pack_start_defaults (GtkBox * box, GtkWidget * child)
   {
-#if GLIB_CHECK_VERSION(2,10,0)
-    g_object_ref_sink (o);
-#else
-    g_object_ref (o);
-    gtk_object_sink (GTK_OBJECT(o));
-#endif
+    gtk_box_pack_start( box, child, TRUE, TRUE, 0 );
   }
 }
 
@@ -87,7 +85,7 @@ namespace
   }
 
   void
-  parent_set_cb (GtkWidget * widget, GtkObject *, gpointer ui_manager_g)
+  parent_set_cb (GtkWidget * widget, GtkWidget *, gpointer ui_manager_g)
   {
     GtkWidget * toplevel = gtk_widget_get_toplevel (widget);
     if (GTK_IS_WINDOW (toplevel))
@@ -110,7 +108,7 @@ namespace
 
   void toggle_visible (GtkWidget * w)
   {
-    set_visible (w, !GTK_WIDGET_VISIBLE(w));
+    set_visible (w, !gtk_widget_get_visible(w));
   }
 }
 
@@ -179,12 +177,8 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
   _connection_size_label (0),
   _queue_size_label (0),
   _queue_size_button (0),
-  _taskbar (0),
-  _ttips (gtk_tooltips_new ())
+  _taskbar (0)
 {
-  g_object_ref_sink_pan (G_OBJECT(_ttips));
-  g_object_weak_ref (G_OBJECT(_root), (GWeakNotify)g_object_unref, _ttips);
-
   char * filename = g_build_filename (file::get_pan_home().c_str(), "pan.ui", NULL);
   if (!gtk_ui_manager_add_ui_from_file (_ui_manager, filename, NULL))
     gtk_ui_manager_add_ui_from_string (_ui_manager, fallback_ui_file, -1, NULL);
@@ -256,9 +250,9 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
   w = _queue_size_label = gtk_label_new (NULL);
   gtk_misc_set_padding (GTK_MISC(w), PAD, 0);
   w = _queue_size_button = gtk_button_new();
-  gtk_tooltips_set_tip (GTK_TOOLTIPS(_ttips), w, _("Open the Task Manager"), NULL);
+  gtk_widget_set_tooltip_text (w, _("Open the Task Manager"));
   gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
-  g_signal_connect (GTK_OBJECT(w), "clicked", G_CALLBACK(show_task_window_cb), this);
+  g_signal_connect (w, "clicked", G_CALLBACK(show_task_window_cb), this);
   gtk_container_add (GTK_CONTAINER(w), _queue_size_label);
   frame = gtk_frame_new (NULL);
   gtk_container_set_border_width (GTK_CONTAINER(frame), 0);
@@ -278,7 +272,7 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
 
   // status 
   w = _event_log_button = gtk_button_new ();
-  gtk_tooltips_set_tip (GTK_TOOLTIPS(_ttips), w, _("Open the Event Log"), NULL);
+  gtk_widget_set_tooltip_text (w, _("Open the Event Log"));
   gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
   gtk_box_pack_start (GTK_BOX(status_bar), w, false, false, 0);
   gtk_container_add (GTK_CONTAINER(w), _info_image);
@@ -294,14 +288,11 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
   _queue.add_listener (this);
   Log::get().add_listener (this);
 
-  gtk_widget_ref (_info_image);
-  gtk_widget_ref (_error_image);
-  gtk_widget_ref (_group_pane->root());
-  gtk_widget_ref (_header_pane->root());
-  gtk_widget_ref (_body_pane->root());
-
-  gtk_object_sink (GTK_OBJECT (_info_image));
-  gtk_object_sink (GTK_OBJECT (_error_image));
+  g_object_ref_sink (G_OBJECT(_info_image));
+  g_object_ref_sink (G_OBJECT(_error_image));
+  g_object_ref (_group_pane->root());
+  g_object_ref (_header_pane->root());
+  g_object_ref (_body_pane->root());
 
   do_work_online (is_action_active ("work-online"));
 
@@ -349,8 +340,8 @@ GUI :: ~GUI ()
   if (vpane)
     _prefs.set_int ("main-window-vpane-position", gtk_paned_get_position(GTK_PANED(vpane)));
 
-  const bool maximized = GTK_WIDGET(_root)->window
-                      && (gdk_window_get_state(_root->window) & GDK_WINDOW_STATE_MAXIMIZED);
+  const bool maximized = gtk_widget_get_window(_root)
+                      && (gdk_window_get_state( gtk_widget_get_window(_root)) & GDK_WINDOW_STATE_MAXIMIZED);
   _prefs.set_flag ("main-window-is-maximized", maximized);
 
   _prefs.remove_listener (this);
@@ -372,7 +363,7 @@ GUI :: ~GUI ()
     delete _views[i];
 
   foreach (std::set<GtkWidget*>, unref, it)
-    gtk_widget_unref (*it);
+    g_object_unref (*it);
   g_object_unref (G_OBJECT(_ui_manager));
 }
 
@@ -384,7 +375,7 @@ void
 GUI :: watch_cursor_on ()
 {
   GdkCursor * cursor = gdk_cursor_new (GDK_WATCH);
-  gdk_window_set_cursor (_root->window, cursor);
+  gdk_window_set_cursor ( gtk_widget_get_window(_root), cursor);
   gdk_cursor_unref (cursor);
   while (gtk_events_pending ())
     gtk_main_iteration ();
@@ -393,7 +384,7 @@ GUI :: watch_cursor_on ()
 void
 GUI :: watch_cursor_off ()
 {
-  gdk_window_set_cursor (_root->window, NULL);
+  gdk_window_set_cursor ( gtk_widget_get_window(_root), NULL);
 }
 
 /***
@@ -505,7 +496,7 @@ GUI :: disable_accelerators_when_focused (GtkWidget * w) const
 
 namespace
 {
-  static std::string prev_path;
+  static std::string prev_path, prev_file;
 }
 
 std::string
@@ -535,6 +526,39 @@ GUI :: prompt_user_for_save_path (GtkWindow * parent, const Prefs& prefs)
   return path;
 }
 
+std::string
+GUI :: prompt_user_for_filename (GtkWindow * parent, const Prefs& prefs)
+{
+	
+  if (prev_path.empty())
+    prev_path = prefs.get_string ("default-save-attachments-path", g_get_home_dir ());
+  if (!file :: file_exists (prev_path.c_str()))
+  prev_path = g_get_home_dir ();
+    prev_file = std::string(_("Untitled.nzb"));
+    
+  GtkWidget * w = gtk_file_chooser_dialog_new (_("Save NZB File as..."),
+				      parent,
+				      GTK_FILE_CHOOSER_ACTION_SAVE,
+				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+				      NULL);
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (w), TRUE);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (w), prev_path.c_str());
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (w), prev_file.c_str());
+	
+	std::string file;
+	const int response (gtk_dialog_run (GTK_DIALOG(w)));
+	if (response == GTK_RESPONSE_ACCEPT) {
+		char *tmp;
+		tmp = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
+		file=tmp;
+		g_free (tmp);
+	}
+	gtk_widget_destroy (w);
+
+  return file;
+}
+
 void GUI :: do_save_articles ()
 {
   std::string path;
@@ -551,6 +575,34 @@ void GUI :: do_save_articles ()
   }
 }
 
+
+void GUI :: do_save_articles_to_nzb ()
+{
+
+  std::string path;
+  const std::vector<const Article*> articles (_header_pane->get_full_selection_v ());
+
+  std::vector<Article> copies;
+  copies.reserve (articles.size());
+  foreach_const (std::vector<const Article*>, articles, it)
+    copies.push_back (**it);
+
+  const std::string file (GUI :: prompt_user_for_filename (get_window(_root), _prefs));
+    if (!file.empty()) {
+      Queue::tasks_t tasks;
+      std::string emptystring;
+      foreach_const (std::vector<Article>, copies, it)
+        tasks.push_back (new TaskArticle (_data, _data, *it, _cache, _data, 0, TaskArticle::RAW,emptystring));
+    
+          // write them to a file
+          std::ofstream tmp(file.c_str());
+          if (tmp.good()) {
+            NZB :: nzb_to_xml_file (tmp, tasks); 
+            tmp.close();
+          }
+    }
+}
+
 namespace
 {
   struct SaveArticlesFromNZB: public Progress::Listener
@@ -567,18 +619,14 @@ namespace
                          ArticleCache& c, const Article& a, const std::string& path):
       _data(d), _queue(q), _root(r), _prefs(p), _cache(c), _article(a), _path(path) {}
 
-    static void foreach_part_cb (GMimeObject *o, gpointer self)
+    static void foreach_part_cb (GMimeObject */*parent*/, GMimeObject *o, gpointer self)
     {
       static_cast<SaveArticlesFromNZB*>(self)->foreach_part (o);
     }
 
     void foreach_part (GMimeObject *o)
     {
-      if (GMIME_IS_MULTIPART (o))
-      {
-        g_mime_multipart_foreach (GMIME_MULTIPART (o), foreach_part_cb, this);
-      }
-      else if (GMIME_IS_PART(o))
+      if (GMIME_IS_PART(o))
       {
         GMimePart * part (GMIME_PART (o));
         GMimeDataWrapper * wrapper (g_mime_part_get_content_object (part));
@@ -591,7 +639,6 @@ namespace
         if (!tasks.empty())
           _queue.add_tasks (tasks, Queue::BOTTOM);
         g_object_unref (mem_stream);
-        g_object_unref (wrapper);
       }
     }
 
@@ -601,7 +648,7 @@ namespace
     {
       if (status == OK) {
         GMimeMessage * message = _cache.get_message (_article.get_part_mids());
-        g_mime_message_foreach_part (message, foreach_part_cb, this);
+        g_mime_message_foreach (message, foreach_part_cb, this);
         g_object_unref (message);
       }
       delete this;
@@ -790,21 +837,21 @@ void GUI :: do_show_profiles_dialog ()
 void GUI :: do_jump_to_group_tab ()
 {
   toggle_action ("tabbed-layout", true);
-  GtkNotebook * n (GTK_NOTEBOOK (GTK_BIN (_workarea_bin)->child));
+  GtkNotebook * n (GTK_NOTEBOOK (gtk_bin_get_child( GTK_BIN (_workarea_bin))));
   gtk_notebook_set_current_page (n, 0);
 }
 
 void GUI :: do_jump_to_header_tab ()
 {
   toggle_action ("tabbed-layout", true);
-  GtkNotebook * n (GTK_NOTEBOOK (GTK_BIN (_workarea_bin)->child));
+  GtkNotebook * n (GTK_NOTEBOOK (gtk_bin_get_child( GTK_BIN (_workarea_bin))));
   gtk_notebook_set_current_page (n, 1);
 }
 
 void GUI :: do_jump_to_body_tab ()
 {
   toggle_action ("tabbed-layout", true);
-  GtkNotebook * n (GTK_NOTEBOOK (GTK_BIN (_workarea_bin)->child));
+  GtkNotebook * n (GTK_NOTEBOOK (gtk_bin_get_child( GTK_BIN (_workarea_bin))));
   gtk_notebook_set_current_page (n, 2);
 }
 
@@ -925,7 +972,7 @@ void GUI :: server_list_dialog_destroyed (GtkWidget *)
   foreach_const (quarks_t, all_servers, it) {
     quarks_t tmp;
     _data.server_get_groups (*it, tmp);
-    if (tmp.empty())
+    if (tmp.empty() && _data.get_server_limits(*it))
       _queue.add_task (new TaskGroups (_data, *it));
   }
 }
@@ -1041,19 +1088,22 @@ void GUI :: do_supersede_article ()
   const char * cpch;
   char * old_mid (g_strdup_printf ("<%s>", g_mime_message_get_message_id(message)));
   GMimeMessage * new_message (g_mime_message_new (false));
-  g_mime_message_set_header (new_message, "Supersedes", old_mid);
+  GMimeObject * new_message_obj = (GMimeObject*)new_message;
+
+  g_mime_object_set_header (new_message_obj, "Supersedes", old_mid);
   g_mime_message_set_sender (new_message, g_mime_message_get_sender (message));
   g_mime_message_set_subject (new_message, g_mime_message_get_subject (message));
-  g_mime_message_set_header (new_message, "Newsgroups", g_mime_message_get_header (message, "Newsgroups"));
-  g_mime_message_set_header (new_message, "References", g_mime_message_get_header (message, "References"));
+  g_mime_object_set_header (new_message_obj, "Newsgroups", g_mime_object_get_header ((GMimeObject *)message, "Newsgroups"));
+  g_mime_object_set_header (new_message_obj, "References", g_mime_object_get_header ((GMimeObject *)message, "References"));
   if ((cpch = g_mime_message_get_reply_to (message)))
               g_mime_message_set_reply_to (new_message, cpch);
-  if ((cpch = g_mime_message_get_header (message,     "Followup-To")))
-              g_mime_message_set_header (new_message, "Followup-To", cpch);
+  if ((cpch = g_mime_object_get_header ((GMimeObject *)message,     "Followup-To")))
+              g_mime_object_set_header (new_message_obj, "Followup-To", cpch);
   gboolean  unused (false);
-  char * body (g_mime_message_get_body (message, true, &unused));
+  char * body (pan_g_mime_message_get_body (message, &unused));
   GMimeStream * stream = g_mime_stream_mem_new_with_buffer (body, strlen(body));
-  GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_DEFAULT);
+  g_free (body);
+  GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_DEFAULT);
   GMimePart * part = g_mime_part_new ();
   g_mime_part_set_content_object (part, content_object);
   g_mime_message_set_mime_part (new_message, GMIME_OBJECT(part));
@@ -1111,11 +1161,11 @@ void GUI :: do_cancel_article ()
   char * cancel_message = g_strdup_printf ("cancel <%s>", g_mime_message_get_message_id(message));
   g_mime_message_set_sender (cancel, g_mime_message_get_sender (message));
   g_mime_message_set_subject (cancel, "Cancel");
-  g_mime_message_set_header (cancel, "Newsgroups", g_mime_message_get_header (message, "Newsgroups"));
-  g_mime_message_set_header (cancel, "Control", cancel_message);
+  g_mime_object_set_header ((GMimeObject *)cancel, "Newsgroups", g_mime_object_get_header ((GMimeObject *)message, "Newsgroups"));
+  g_mime_object_set_header ((GMimeObject *)cancel, "Control", cancel_message);
   const char * body ("Ignore\r\nArticle canceled by author using " PACKAGE_STRING "\r\n");
   GMimeStream * stream = g_mime_stream_mem_new_with_buffer (body, strlen(body));
-  GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_DEFAULT);
+  GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_DEFAULT);
   GMimePart * part = g_mime_part_new ();
   g_mime_part_set_content_object (part, content_object);
   g_mime_message_set_mime_part (cancel, GMIME_OBJECT(part));
@@ -1194,12 +1244,14 @@ GUI :: do_post ()
       newsgroups = group;
   }
   if (!newsgroups.empty())
-    g_mime_message_add_header (message, "Newsgroups", newsgroups.c_str());
+    g_mime_object_append_header ((GMimeObject *) message, "Newsgroups", newsgroups.c_str());
 
   // content type
   GMimePart * part = g_mime_part_new ();
-  g_mime_part_set_content_type (part, g_mime_content_type_new_from_string ("text/plain; charset=UTF-8"));
-  g_mime_part_set_encoding (part, GMIME_PART_ENCODING_8BIT);
+  GMimeContentType *type = g_mime_content_type_new_from_string ("text/plain; charset=UTF-8");
+  g_mime_object_set_content_type ((GMimeObject *) part, type);
+  g_object_unref (type);
+  g_mime_part_set_content_encoding (part, GMIME_CONTENT_ENCODING_8BIT);
   g_mime_message_set_mime_part (message, GMIME_OBJECT(part));
   g_object_unref (part);
 
@@ -1244,14 +1296,13 @@ void GUI :: do_tip_jar ()
 }
 void GUI :: do_about_pan ()
 {
-#if GTK_CHECK_VERSION(2,6,0)
-  const gchar * authors [] = { "Charles Kerr <charles@rebelbase.com>", "Calin Culianu <calin@ajvar.org> - Threaded Decoding", 0 };
+  const gchar * authors [] = { "Charles Kerr <charles@rebelbase.com> - Pan Author", "Calin Culianu <calin@ajvar.org> - Threaded Decoding", "K. Haley <haleykd@users.sf.net> - Contributor", "Petr Kovar <pknbe@volny.cz> - Contributor", "Heinrich Mueller <eddie_v@gmx.de> - Contributor", "Christophe Lambin <chris@rebelbase.com> - Original Pan Development", "Matt Eagleson <matt@rebelbase.com> - Original Pan Development", 0 };
   GdkPixbuf * logo = gdk_pixbuf_new_from_inline(-1, icon_pan_about_logo, 0, 0);
   GtkAboutDialog * w (GTK_ABOUT_DIALOG (gtk_about_dialog_new ()));
-  gtk_about_dialog_set_name (w, _("Pan"));
+  gtk_about_dialog_set_program_name (w, _("Pan"));
   gtk_about_dialog_set_version (w, PACKAGE_VERSION);
-  gtk_about_dialog_set_comments (w, VERSION_TITLE);
-  gtk_about_dialog_set_copyright (w, _("Copyright © 2002-2007 Charles Kerr"));
+  gtk_about_dialog_set_comments (w, VERSION_TITLE " (" GIT_REV "; " PLATFORM_INFO ")");
+  gtk_about_dialog_set_copyright (w, _("Copyright © 2002-2011 Charles Kerr and others"));
   gtk_about_dialog_set_website (w, "http://pan.rebelbase.com/");
   gtk_about_dialog_set_logo (w, logo);
   gtk_about_dialog_set_license (w, LICENSE);
@@ -1260,23 +1311,6 @@ void GUI :: do_about_pan ()
   g_signal_connect (G_OBJECT (w), "response", G_CALLBACK (gtk_widget_destroy), NULL);
   gtk_widget_show_all (GTK_WIDGET(w));
   g_object_unref (logo);
-#else
-  GtkWidget * dialog = gtk_dialog_new_with_buttons (PACKAGE_STRING,
-                                                    GTK_WINDOW (get_window (_root)),
-                                                    GtkDialogFlags (GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
-                                                    GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-                                                    NULL);
-  GdkPixbuf * logo = gdk_pixbuf_new_from_inline(-1, icon_pan_about_logo, 0, 0);
-  GtkBox * box = GTK_BOX(GTK_DIALOG(dialog)->vbox);
-  gtk_box_pack_start (box, gtk_image_new_from_pixbuf (logo), false, false, PAD);
-  gtk_box_pack_start (box, gtk_label_new("Pan " PACKAGE_VERSION), false, false, PAD);
-  gtk_box_pack_start (box, gtk_label_new(VERSION_TITLE), false, false, 0);
-  gtk_box_pack_start (box, gtk_label_new(_("Copyright © 2002-2007 Charles Kerr")), false, false, 0);
-  gtk_box_pack_start (box, gtk_label_new("http://pan.rebelbase.com/"), false, false, PAD);
-  gtk_widget_show_all (dialog);
-  g_signal_connect_swapped (dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
-  g_object_unref (logo);
-#endif
 }
 
 void GUI :: do_work_online (bool b)
@@ -1289,8 +1323,9 @@ namespace
 {
   void remove_from_parent (GtkWidget * w)
   {
-    if (w->parent != 0)
-      gtk_container_remove (GTK_CONTAINER(w->parent), w);
+    GtkWidget *parent = gtk_widget_get_parent(w);
+    if (parent != 0)
+      gtk_container_remove (GTK_CONTAINER(parent), w);
   }
 
   enum { HORIZONTAL, VERTICAL };
@@ -1365,7 +1400,8 @@ void GUI :: do_tabbed_layout (bool tabbed)
     vpane = 0;
   }
 
-  gtk_widget_hide_all (_workarea_bin);
+  //gtk_widget_hide_all (_workarea_bin);
+  gtk_widget_hide (_workarea_bin);
 
   GtkWidget * group_w (_group_pane->root());
   GtkWidget * header_w (_header_pane->root());
@@ -1376,7 +1412,7 @@ void GUI :: do_tabbed_layout (bool tabbed)
   remove_from_parent (body_w);
 
   // remove workarea's current child
-  GList * children = gtk_container_children (GTK_CONTAINER(_workarea_bin));
+  GList * children = gtk_container_get_children (GTK_CONTAINER(_workarea_bin));
   if (children) {
     gtk_container_remove (GTK_CONTAINER(_workarea_bin), GTK_WIDGET(children->data));
     g_list_free (children);
@@ -1390,7 +1426,6 @@ void GUI :: do_tabbed_layout (bool tabbed)
     gtk_notebook_append_page (n, group_w, gtk_label_new_with_mnemonic (_("_1. Group Pane")));
     gtk_notebook_append_page (n, header_w, gtk_label_new_with_mnemonic (_("_2. Header Pane")));
     gtk_notebook_append_page (n, body_w, gtk_label_new_with_mnemonic (_("_3. Body Pane")));
-    gtk_notebook_set_tab_border (n, PAD_SMALL);
     g_signal_connect (n, "switch-page", G_CALLBACK(notebook_page_switched_cb), this);
   }
   else
@@ -1429,7 +1464,7 @@ void GUI :: do_tabbed_layout (bool tabbed)
   set_visible (body_w, is_action_active("show-body-pane"));
 
   if (tabbed)
-    gtk_notebook_set_current_page (GTK_NOTEBOOK(GTK_BIN(_workarea_bin)->child), 0);
+    gtk_notebook_set_current_page (GTK_NOTEBOOK( gtk_bin_get_child( GTK_BIN(_workarea_bin))), 0);
 }
 
 void GUI :: do_show_group_pane (bool b)
@@ -1618,7 +1653,8 @@ void GUI :: do_refresh_groups ()
 
   const quarks_t servers (_data.get_servers ());
   foreach_const_r (quarks_t, servers, it)
-    tasks.push_back (new TaskGroups (_data, *it));
+    if (_data.get_server_limits(*it))
+      tasks.push_back (new TaskGroups (_data, *it));
 
   if (!tasks.empty())
     _queue.add_tasks (tasks);
@@ -1641,10 +1677,10 @@ void GUI :: do_prompt_for_charset ()
   if (_charset.empty())
       _charset = "UTF-8";
 
-  char * tmp = e_charset_picker_dialog (_("Character Encoding"),
-                                        _("Body Pane Encoding"),
-                                        _charset.c_str(),
-                                        get_window(root()));
+  char * tmp = e_charset_dialog (_("Character Encoding"),
+                                 _("Body Pane Encoding"),
+                                 _charset.c_str(),
+                                 get_window(root()));
   set_charset (tmp);
   free (tmp);
 }
@@ -1711,7 +1747,7 @@ GUI :: refresh_connection_label ()
   }
 
   gtk_label_set_text (GTK_LABEL(_connection_size_label), str);
-  gtk_tooltips_set_tip (GTK_TOOLTIPS(_ttips), _connection_size_eventbox, tip, NULL);
+  gtk_widget_set_tooltip_text (_connection_size_eventbox, tip);
 }
 
 namespace
@@ -1744,8 +1780,8 @@ void
 GUI :: set_queue_size_label (unsigned int running,
                              unsigned int size)
 {
-  char str[128];
-  char tip[128];
+  char str[256];
+  char tip[256];
 
   // build the button label
   if (!size)
@@ -1766,7 +1802,7 @@ GUI :: set_queue_size_label (unsigned int running,
 
   // update the gui
   gtk_label_set_text (GTK_LABEL(_queue_size_label), str);
-  gtk_tooltips_set_tip (GTK_TOOLTIPS(_ttips), _queue_size_button, tip, NULL);
+  gtk_widget_set_tooltip_text (_queue_size_button, tip);
 }
 
 void
