@@ -30,6 +30,8 @@
 #include <pan/gui/group-prefs.h>
 #include <pan/gui/wait.h>
 
+#include "gtk_compat.h"
+
 namespace pan
 {
   class GroupPane;
@@ -73,6 +75,7 @@ namespace pan
       virtual void do_prompt_for_charset ();
       virtual void do_save_articles ();
       virtual void do_save_articles_from_nzb ();
+      virtual void do_save_articles_to_nzb ();
       virtual void do_print ();
       virtual void do_quit ();
       virtual void do_import_tasks ();
@@ -153,7 +156,7 @@ namespace pan
 
     public:
       static std::string prompt_user_for_save_path (GtkWindow * parent, const Prefs& prefs);
-
+	  static std::string prompt_user_for_filename  (GtkWindow * parent, const Prefs& prefs);
 
     private: // Queue::Listener
       friend class Queue;
@@ -209,7 +212,6 @@ namespace pan
       GtkWidget * _taskbar;
       std::vector<ProgressView*> _views;
       std::list<Task*> _active_tasks;
-      GtkTooltips * _ttips;
 
       std::string _charset;
 
