@@ -36,8 +36,8 @@ extern "C" {
 #include <pan/icons/pan-pixbufs.h>
 #include "header-pane.h"
 #include "render-bytes.h"
-#include "sexy-icon-entry.h"
 #include "tango-colors.h"
+#include "gtk_compat.h"
 
 using namespace pan;
 
@@ -362,8 +362,10 @@ HeaderPane :: column_compare_func (GtkTreeModel  * model,
 {
   int ret (0);
   const PanTreeStore * store (reinterpret_cast<PanTreeStore*>(model));
-  const Row& row_a (*dynamic_cast<const Row*>(store->get_row (iter_a)));
-  const Row& row_b (*dynamic_cast<const Row*>(store->get_row (iter_b)));
+  //const Row& row_a (*dynamic_cast<const Row*>(store->get_row (iter_a)));
+  //const Row& row_b (*dynamic_cast<const Row*>(store->get_row (iter_b)));
+  const Row& row_a (*static_cast<const Row*>(store->get_row (iter_a)));
+  const Row& row_b (*static_cast<const Row*>(store->get_row (iter_b)));
 
   int sortcol;
   GtkSortType order;
@@ -544,8 +546,7 @@ HeaderPane :: set_group (const Quark& new_group)
       _atree->add_listener (this);
 
       rebuild ();
-
-      if (GTK_WIDGET_REALIZED(_tree_view))
+      if (gtk_widget_get_realized(_tree_view))
         gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(_tree_view), 0, 0);
     }
   }
@@ -648,7 +649,6 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
   // if the old selection survived,
   // is it visible on the screen?
   bool selection_was_visible (true);
-#if GTK_CHECK_VERSION(2,8,0)
   if (!new_selection.empty()) {
     GtkTreeView *view (GTK_TREE_VIEW(_tree_view));
     Row * row (get_row (*new_selection.begin()));
@@ -660,7 +660,6 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
     gtk_tree_path_free (b);
     gtk_tree_path_free (p);
   }
-#endif
 
   // if none of the current selection survived,
   // we need to select something to replace the
@@ -1011,6 +1010,7 @@ namespace
   const char * mode_strings [] =
   {
     N_("Subject or Author"),
+    N_("Sub or Auth (regex)"),
     N_("Subject"),
     N_("Author"),
     N_("Message-ID"),
@@ -1019,6 +1019,7 @@ namespace
   enum
   {
     SUBJECT_OR_AUTHOR,
+    SUBJECT_OR_AUTHOR_REGEX,
     SUBJECT,
     AUTHOR,
     MESSAGE_ID
@@ -1054,6 +1055,14 @@ HeaderPane :: rebuild_filter (const std::string& text, int mode)
       f2.set_type_text ("From", d);
       entry_filter._aggregates.push_back (f1);
       entry_filter._aggregates.push_back (f2);
+    } else if (mode == SUBJECT_OR_AUTHOR_REGEX) {
+      FilterInfo f1, f2;
+      entry_filter.set_type_aggregate_or ();
+      d.type = TextMatch::REGEX;
+      f1.set_type_text ("Subject", d);
+      f2.set_type_text ("From", d);
+      entry_filter._aggregates.push_back (f1);
+      entry_filter._aggregates.push_back (f2);
     }
     f._aggregates.push_back (entry_filter);
   }
@@ -1187,19 +1196,29 @@ namespace
                                      GdkEventFocus * ,
                                      gpointer        )
   {
+#if !GTK_CHECK_VERSION(3,0,0)
     gtk_widget_modify_text (w, GTK_STATE_NORMAL, NULL); // resets
+#else
+    gtk_widget_override_color (w, GTK_STATE_FLAG_NORMAL, NULL);
+#endif
     set_search_entry (w, search_text.c_str());
     return false;
   }
 
   void refresh_search_entry (GtkWidget * w)
   {
-    if (search_text.empty() && !GTK_WIDGET_HAS_FOCUS(w))
+    if (search_text.empty() && !gtk_widget_has_focus(w))
     {
+#if !GTK_CHECK_VERSION(3,0,0)
       GdkColor c;
       c.pixel = 0;
       c.red = c.green = c.blue = 0xAAAA;
       gtk_widget_modify_text (w, GTK_STATE_NORMAL, &c);
+#else
+      GdkRGBA c;
+      gdk_rgba_parse (&c, "0xAAA");
+      gtk_widget_override_color(w, GTK_STATE_FLAG_NORMAL, &c);
+#endif
       set_search_entry (w, _(mode_strings[mode]));
     }
   }
@@ -1271,15 +1290,15 @@ namespace
     }
   }
 
-  void entry_icon_released (SexyIconEntry*, SexyIconEntryPosition icon_pos, int, gpointer menu)
+  void entry_icon_release (GtkEntry*, GtkEntryIconPosition icon_pos, GdkEventButton*, gpointer menu)
   {
-    if (icon_pos == SEXY_ICON_ENTRY_PRIMARY)
+    if (icon_pos == GTK_ENTRY_ICON_PRIMARY)
       gtk_menu_popup (GTK_MENU(menu), 0, 0, 0, 0, 0, gtk_get_current_event_time());
   }
 
-  void entry_icon_released_2 (SexyIconEntry *entry, SexyIconEntryPosition icon_pos, int, gpointer pane_gpointer)
+  void entry_icon_release_2 (GtkEntry *entry, GtkEntryIconPosition icon_pos, GdkEventButton*, gpointer pane_gpointer)
   {
-    if (icon_pos == SEXY_ICON_ENTRY_SECONDARY) {
+    if (icon_pos == GTK_ENTRY_ICON_SECONDARY) {
       set_search_entry (GTK_WIDGET(entry), "");
       refresh_search_entry (GTK_WIDGET(entry));
       search_text.clear ();
@@ -1290,9 +1309,7 @@ namespace
 
   void ellipsize_if_supported (GObject * o)
   {
-#if GTK_CHECK_VERSION(2,6,0)
     g_object_set (o, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
-#endif
   }
 }
 
@@ -1489,7 +1506,7 @@ HeaderPane :: ~HeaderPane ()
 GtkWidget*
 HeaderPane :: create_filter_entry ()
 {
-  GtkWidget * entry = sexy_icon_entry_new ();
+  GtkWidget * entry = gtk_entry_new ();
   _action_manager.disable_accelerators_when_focused (entry);
   g_object_set_data (G_OBJECT(entry), "header-pane", this);
   g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), NULL);
@@ -1497,27 +1514,32 @@ HeaderPane :: create_filter_entry ()
   g_signal_connect (entry, "activate", G_CALLBACK(search_entry_activated), this);
   entry_changed_tag = g_signal_connect (entry, "changed", G_CALLBACK(search_entry_changed), this);
 
-  GtkWidget * image = gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
-  sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, GTK_IMAGE(image));
-  sexy_icon_entry_set_icon_highlight(SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, true);
-
-  image = gtk_image_new_from_stock ("ICON_SEARCH_PULLDOWN", GTK_ICON_SIZE_MENU);
-  sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_PRIMARY, GTK_IMAGE(image));
-  sexy_icon_entry_set_icon_highlight (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_PRIMARY, TRUE);
+      gtk_entry_set_icon_from_stock( GTK_ENTRY( entry ),
+                                   GTK_ENTRY_ICON_PRIMARY,
+                                   GTK_STOCK_FIND);
+   gtk_entry_set_icon_from_stock( GTK_ENTRY( entry ),
+                                  GTK_ENTRY_ICON_SECONDARY,
+                                  GTK_STOCK_CLEAR );
 
+  bool regex = _prefs.get_flag ("use-regex", false);
   GtkWidget * menu = gtk_menu_new ();
-  mode = 0;
+  if (regex == true )
+    mode = 1;
+  else
+    mode = 0;
   GSList * l = 0;
   for (int i=0, qty=G_N_ELEMENTS(mode_strings); i<qty; ++i) {
     GtkWidget * w = gtk_radio_menu_item_new_with_label (l, _(mode_strings[i]));
     l = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM(w));
     g_object_set_data (G_OBJECT(w), "MODE", GINT_TO_POINTER(i));
     g_signal_connect (w, "toggled", G_CALLBACK(search_menu_toggled_cb),entry);
+    if (mode == i)
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(w), TRUE);
     gtk_menu_shell_append (GTK_MENU_SHELL(menu), w);
     gtk_widget_show (w);
   }
-  g_signal_connect (entry, "icon-released", G_CALLBACK(entry_icon_released), menu);
-  g_signal_connect (entry, "icon-released", G_CALLBACK(entry_icon_released_2), this);
+  g_signal_connect (entry, "icon-release", G_CALLBACK(entry_icon_release), menu);
+  g_signal_connect (entry, "icon-release", G_CALLBACK(entry_icon_release_2), this);
 
   refresh_search_entry (entry);
 
@@ -1542,6 +1564,7 @@ HeaderPane :: on_selection_changed_idle (gpointer self_gpointer)
     "download-selected-article",
     "save-articles",
     "save-articles-from-nzb",
+    "save-articles-to-nzb",
     "read-selected-article",
     "show-selected-article-info",
     "mark-article-read",
@@ -1600,12 +1623,8 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
   // build the view...
   GtkWidget * w = _tree_view = gtk_tree_view_new ();
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW(w), false);
-#if GTK_CHECK_VERSION(2,8,0)
   gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW(w), true);
-#endif
-#if GTK_CHECK_VERSION(2,10,0)
   gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW(w), true);
-#endif
 
   GtkTreeSelection * sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(w));
   gtk_tree_selection_set_mode (sel, GTK_SELECTION_MULTIPLE);
@@ -1991,11 +2010,11 @@ void
 HeaderPane :: refresh_font ()
 {
   if (!_prefs.get_flag ("header-pane-font-enabled", false))
-    gtk_widget_modify_font (_tree_view, 0);
+    gtk_widget_override_font (_tree_view, 0);
   else {
     const std::string str (_prefs.get_string ("header-pane-font", "Sans 10"));
     PangoFontDescription * pfd (pango_font_description_from_string (str.c_str()));
-    gtk_widget_modify_font (_tree_view, pfd);
+    gtk_widget_override_font (_tree_view, pfd);
     pango_font_description_free (pfd);
   }
 }
@@ -2052,6 +2071,9 @@ HeaderPane :: on_queue_task_removed (Queue&, Task& task, int)
 void
 HeaderPane :: on_cache_added (const Quark& message_id)
 {
+  quarks_t q;
+  q.insert(message_id);
+  _data.rescore_articles ( _group, q );
   rebuild_article_action (message_id);
 }
 void
