@@ -62,16 +62,16 @@ const char * fallback_ui_file =
 "        <menuitem action='wrap-article-body' />\n"
 "        <menuitem action='mute-quoted-text' />\n"
 "        <menuitem action='show-all-headers' />\n"
-"        <menuitem action='rot13-selected-text' />\n"
 "        <menuitem action='monospace-font-enabled' />\n"
 "        <menuitem action='show-smilies-as-graphics' />\n"
 "        <menuitem action='show-text-markup'/>\n"
+"        <menuitem action='highlight-urls'/>\n"
+"        <menuitem action='rot13-selected-text' />\n"
+"        <separator />\n"
 "        <menuitem action='size-pictures-to-fit'/>\n"
 "        <menuitem action='focus-on-image'/>\n"
-"      </menu>\n"
-"      <separator />\n"
-"      <menu action='character-encoding-menu'>\n"
-"        <placeholder name='character-encoding-placeholder' />\n"
+"        <separator />\n"
+"        <menuitem action='set-charset' />\n"
 "      </menu>\n"
 "    </menu>\n"
 "    <menu action='go-menu'>\n"
@@ -113,6 +113,8 @@ const char * fallback_ui_file =
 "      </menu>\n"
 "      <menu action='article-actions-menu'>\n"
 "        <menuitem action='save-articles' />\n"
+"        <menuitem action='save-articles-from-nzb' />\n"
+"        <menuitem action='save-articles-to-nzb' />\n"
 "        <separator />\n"
 "        <menuitem action='read-selected-article' />\n"
 "        <menuitem action='download-selected-article' />\n"
@@ -137,7 +139,6 @@ const char * fallback_ui_file =
 "        <menuitem action='supersede-article' />\n"
 "        <menuitem action='cancel-article' />\n"
 "      </menu>\n"
-//"    </menu>\n"
 "    <menu action='help-menu'>\n"
 "      <menuitem action='pan-web-page' />\n"
 "      <menuitem action='tip-jar' />\n"
@@ -174,6 +175,8 @@ const char * fallback_ui_file =
 "\n"
 "  <popup name='header-pane-popup'>\n"
 "    <menuitem action='save-articles' />\n"
+"    <menuitem action='save-articles-from-nzb' />\n"
+"    <menuitem action='save-articles-to-nzb' />\n"
 "    <separator />\n"
 "    <menuitem action='read-selected-article' />\n"
 "    <menuitem action='download-selected-article' />\n"
