@@ -21,6 +21,7 @@
 #include <fstream>
 #include <iostream>
 #include <sstream>
+#include <unistd.h>
 extern "C" {
   #include <gmime/gmime.h>
   #include <glib/gi18n.h>
@@ -39,13 +40,14 @@ extern "C" {
 #include <pan/usenet-utils/mime-utils.h>
 #include <pan/data/data.h>
 #include <pan/tasks/task-post.h>
-#include "e-charset-picker.h"
+#include "e-charset-dialog.h"
 #include "pad.h"
 #include "hig.h"
 #include "post-ui.h"
 #include "post.ui.h"
 #include "profiles-dialog.h"
 #include "url.h"
+#include "gtk_compat.h"
 
 #ifdef HAVE_GTKSPELL
 #define DEFAULT_SPELLCHECK_FLAG true
@@ -53,17 +55,18 @@ extern "C" {
 #define DEFAULT_SPELLCHECK_FLAG false
 #endif
 
-#if !GTK_CHECK_VERSION(2,6,0)
-#define GTK_STOCK_EDIT GTK_STOCK_OPEN
-#endif
-
 using namespace pan;
 
 #define USER_AGENT_PREFS_KEY "add-user-agent-header-when-posting"
 #define MESSAGE_ID_PREFS_KEY "add-message-id-header-when-posting"
+#define USER_AGENT_EXTRA_PREFS_KEY "user-agent-extra-info"
 
 namespace
 {
+#ifndef HAVE_CLOSE
+inline int close(int fd) {return _close(fd);}
+#endif
+
   bool remember_charsets (true);
 
   void on_remember_charset_toggled (GtkToggleAction * toggle, gpointer)
@@ -131,7 +134,8 @@ PostUI :: get_body () const
   std::string body;
   GtkTextBuffer * buf (_body_buf);
   GtkTextView * view (GTK_TEXT_VIEW(_body_view));
-
+  const bool wrap (_prefs.get_flag ("compose-wrap-enabled", false));
+  
   // walk through all the complete lines...
   GtkTextIter body_start, body_end, line_start, line_end;
   gtk_text_buffer_get_bounds (buf, &body_start, &body_end);
@@ -140,7 +144,7 @@ PostUI :: get_body () const
     char * line = gtk_text_buffer_get_text (buf, &line_start, &line_end, false);
     body += line;
     g_free (line);
-    if (*body.rbegin() != '\n')
+    if (wrap && *body.rbegin() != '\n')
       body += '\n';
     line_start = line_end;
   }
@@ -167,9 +171,9 @@ PostUI :: set_wrap_mode (bool wrap)
 
   if (_body_buf) {
     const std::string s (get_body());
-    gtk_text_buffer_set_text (_body_buf, s.c_str(), s.size());
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(_body_view),
                                  wrap ? GTK_WRAP_WORD : GTK_WRAP_NONE);
+    gtk_text_buffer_set_text (_body_buf, s.c_str(), s.size());
   }
 }
 
@@ -194,6 +198,7 @@ namespace
   void do_close    (GtkAction*, gpointer p) { static_cast<PostUI*>(p)->close_window (); }
   void do_wrap     (GtkToggleAction * w, gpointer p) { static_cast<PostUI*>(p)->set_wrap_mode (gtk_toggle_action_get_active (w)); }
   void do_edit2    (GtkToggleAction * w, gpointer p) { static_cast<PostUI*>(p)->set_always_run_editor (gtk_toggle_action_get_active (w)); }
+  void do_wrap_selected(GtkAction*, gpointer p) { static_cast<PostUI*>(p)->wrap_selection(); }
 
   GtkActionEntry entries[] =
   {
@@ -211,13 +216,14 @@ namespace
     { "copy", GTK_STOCK_COPY, 0, 0, 0, G_CALLBACK(do_copy) },
     { "paste", GTK_STOCK_PASTE, 0, 0, 0, G_CALLBACK(do_paste) },
     { "rot13", GTK_STOCK_REFRESH, N_("_Rot13"), 0, N_("Rot13 Selected Text"), G_CALLBACK(do_rot13) },
-    { "run-editor", GTK_STOCK_JUMP_TO, N_("Run _Editor"), "<control>e", 0, G_CALLBACK(do_edit) },
-    { "manage-profiles", GTK_STOCK_EDIT, N_("Edit P_osting Profiles"), 0, 0, G_CALLBACK(do_profiles) }
+    { "run-editor", GTK_STOCK_JUMP_TO, N_("Run _Editor"), "<control>e", N_("Run Editor"), G_CALLBACK(do_edit) },
+    { "manage-profiles", GTK_STOCK_EDIT, N_("Edit P_osting Profiles"), 0, 0, G_CALLBACK(do_profiles) },
+    { "wrapselected", 0, N_("Wrap Selected"), 0, 0, G_CALLBACK(do_wrap_selected) }
   };
 
   GtkToggleActionEntry toggle_entries[] =
   {
-    { "wrap", GTK_STOCK_JUSTIFY_FILL, N_("_Wrap Text"), 0, 0, G_CALLBACK(do_wrap), true },
+    { "wrap", GTK_STOCK_JUSTIFY_FILL, N_("_Wrap Text"), 0, N_("Wrap Text"), G_CALLBACK(do_wrap), true },
     { "always-run-editor", 0, N_("Always Run Editor"), 0, 0, G_CALLBACK(do_edit2), false },
     { "remember-charset", 0, N_("Remember Character Encoding for this Group"), 0, 0, G_CALLBACK(on_remember_charset_toggled), true },
     { "spellcheck", 0, N_("Check _Spelling"), 0, 0, G_CALLBACK(on_spellcheck_toggled), true }
@@ -244,10 +250,10 @@ PostUI :: prompt_for_charset ()
   if (_charset.empty())
       _charset = DEFAULT_CHARSET;
 
-  char * tmp = e_charset_picker_dialog (_("Character Encoding"),
-                                        _("New Article's Encoding:"),
-                                        _charset.c_str(),
-                                        GTK_WINDOW(root()));
+  char * tmp = e_charset_dialog (_("Character Encoding"),
+                                 _("New Article's Encoding:"),
+                                 _charset.c_str(),
+                                 GTK_WINDOW(root()));
   set_charset (tmp);
   free (tmp);
 }
@@ -316,6 +322,21 @@ PostUI :: rot13_selection ()
   }
 } 
 
+void
+PostUI :: wrap_selection ()
+{
+  GtkTextIter start, end;
+  if (gtk_text_buffer_get_selection_bounds (_body_buf, &start, &end))
+  {
+    char * str (gtk_text_buffer_get_text (_body_buf, &start, &end, false));
+    std::string s(str);
+    s = _tm.fill(s);
+    gtk_text_buffer_delete (_body_buf, &start, &end);
+    gtk_text_buffer_insert (_body_buf, &start, s.c_str(), s.length() );
+    g_free (str);
+  }
+}
+
 namespace
 {
   gboolean delete_event_cb (GtkWidget*, GdkEvent*, gpointer user_data)
@@ -350,6 +371,11 @@ PostUI :: close_window ()
     gtk_widget_destroy (d);
   }
 
+  int w,h;
+  gtk_window_get_size( GTK_WINDOW(_root), &w, &h);
+  _prefs.set_int("post-ui-width", w);
+  _prefs.set_int("post-ui-height", h);
+
   if (destroy_flag)
     gtk_widget_destroy (_root);
 }
@@ -459,21 +485,6 @@ namespace
   }
 }
 
-namespace
-{
-#if !GTK_CHECK_VERSION(2,6,0)
-  char* gtk_combo_box_get_active_text (GtkComboBox * combo_box) {
-    char * text (0);
-    GtkTreeIter  iter;
-    if (gtk_combo_box_get_active_iter (combo_box, &iter)) {
-      GtkTreeModel * model = gtk_combo_box_get_model (combo_box);
-      gtk_tree_model_get (model, &iter, 0, &text, -1);
-    }
-    return text;
-  }
-#endif
-}
-
 void
 PostUI :: send_now ()
 {
@@ -515,8 +526,9 @@ namespace
       if (keep.count(*pch))
         out += *pch;
       else {
+        unsigned char uc = (unsigned char)*pch;
         char buf[8];
-        g_snprintf (buf, sizeof(buf), "%%%02x", (int)*pch);
+        g_snprintf (buf, sizeof(buf), "%%%02x", (int)uc);
         out += buf;
       }
     }
@@ -529,8 +541,8 @@ PostUI :: maybe_mail_message (GMimeMessage * message)
 {
   std::string url, to, groups;
   gboolean unused;
-  char * headers (g_mime_message_get_headers (message));
-  char * body (g_mime_message_get_body (message, true, &unused));
+  char * headers (g_mime_object_get_headers ((GMimeObject *) message));
+  char * body (pan_g_mime_message_get_body (message, &unused));
   StringView key, val, v(headers);
   v.trim ();
   while (v.pop_token (val, '\n') && val.pop_token(key,':')) {
@@ -629,7 +641,7 @@ PostUI :: maybe_post_message (GMimeMessage * message)
   *** If this is email only, skip the rest of the posting...
   *** we only stayed this long to get check_message()
   **/
-  const StringView groups (g_mime_message_get_header (message, "Newsgroups"));
+  const StringView groups (g_mime_object_get_header ((GMimeObject *) message, "Newsgroups"));
   if (groups.empty()) {
     maybe_mail_message (message);
     return true;
@@ -667,11 +679,12 @@ PostUI :: maybe_post_message (GMimeMessage * message)
   char buf[512];
   g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Posting..."));
   GtkWidget * w = GTK_WIDGET (g_object_new (GTK_TYPE_LABEL, "use-markup", TRUE, "label", buf, NULL));
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d)->vbox), w, false, false, PAD_SMALL);
+  GtkWidget *content = gtk_dialog_get_content_area(GTK_DIALOG(d));
+  gtk_box_pack_start (GTK_BOX(content), w, false, false, PAD_SMALL);
   w = gtk_progress_bar_new ();
   gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR(w), 0.05);
   const guint tag = g_timeout_add (100, pulse_me, w);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d)->vbox), w, false, false, PAD_SMALL);
+  gtk_box_pack_start (GTK_BOX(content), w, false, false, PAD_SMALL);
   g_object_set_data_full (G_OBJECT(d), "progressbar-timeout-tag", GUINT_TO_POINTER(tag), remove_progress_tag);
   _post_dialog = d;
   g_signal_connect (_post_dialog, "destroy", G_CALLBACK(gtk_widget_destroyed), &_post_dialog);
@@ -701,10 +714,27 @@ PostUI :: maybe_post_message (GMimeMessage * message)
 ****
 ***/
 
+namespace
+{
+	typedef struct
+	{
+		GPid pid;
+		char *fname;
+		PostUI *pui;
+	} se_data;
+	
+	void child_watch_cb(GPid pid, gint status, gpointer data)
+	{
+		se_data *d=static_cast<se_data*>(data);
+		static_cast<PostUI*>(d->pui)->spawn_editor_dead(static_cast<char*>(d->fname));
+		g_spawn_close_pid(pid);
+		delete d;
+	}
+}
+
 void
 PostUI :: spawn_editor ()
 {
-  GtkTextBuffer * buf (_body_buf);
   bool ok (true);
 
   // open a new tmp file
@@ -714,8 +744,10 @@ PostUI :: spawn_editor ()
     GError * err = NULL;
     const int fd (g_file_open_tmp ("pan_edit_XXXXXX", &fname, &err));
     if (fd != -1)
-      fp = fdopen (fd, "w");
-    else {
+    {
+      close(fd);
+      fp = g_fopen (fname, "w");
+    } else {
       Log::add_err (err && err->message ? err->message : _("Error opening temporary file"));
       if (err)
         g_clear_error (&err);
@@ -783,17 +815,32 @@ PostUI :: spawn_editor ()
   // spawn off the external editor
   if (ok) {
     GError * err (0);
-    g_spawn_sync (0, argv, 0, G_SPAWN_SEARCH_PATH, 0, 0, 0, 0, 0, &err);
+    se_data *data=new se_data;
+    data->fname=fname;
+    data->pui=this;
+    g_spawn_async (0, argv, 0, (GSpawnFlags)(G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD), 0, 0, &data->pid, &err);
     if (err != NULL) {
       Log::add_err_va (_("Error starting external editor: %s"), err->message);
       g_clear_error (&err);
       ok = false;
+      delete data;
+    } else {
+      g_child_watch_add(data->pid,child_watch_cb,static_cast<gpointer>(data));
     }
+  } else {
+	  g_free(fname);
   }
 
+  g_strfreev (argv);
+}
+
+void PostUI::spawn_editor_dead(char *fname)
+{
+	GtkTextBuffer * buf (_body_buf);
+	
   // read the file contents back in
   std::string txt;
-  if (ok && file :: get_text_file_contents (fname, txt)) {
+  if (file :: get_text_file_contents (fname, txt)) {
     GtkTextIter start, end;
     gtk_text_buffer_get_bounds (buf, &start, &end);
     gtk_text_buffer_delete (buf, &start, &end);
@@ -803,7 +850,6 @@ PostUI :: spawn_editor ()
   // cleanup
   ::remove (fname);
   g_free (fname);
-  g_strfreev (argv);
 
   gtk_window_present (GTK_WINDOW(root()));
 }
@@ -869,9 +915,14 @@ PostUI :: open_draft ()
 
 namespace
 {
+  bool ua_extra=false;
+
   const char * get_user_agent ()
   {
-    return "Pan/" PACKAGE_VERSION " (" VERSION_TITLE ")";
+    if (ua_extra)
+      return "Pan/" PACKAGE_VERSION " (" VERSION_TITLE "; " GIT_REV "; " PLATFORM_INFO ")";
+    else
+      return "Pan/" PACKAGE_VERSION " (" VERSION_TITLE "; " GIT_REV ")";
   }
 
   bool header_has_dedicated_entry (const StringView& name)
@@ -903,9 +954,9 @@ namespace
    */
   void pan_g_mime_message_set_message_id (GMimeMessage *msg, const char *mid)
   {
-    g_mime_message_add_header (msg, "Message-ID", mid);
+    g_mime_object_append_header ((GMimeObject *) msg, "Message-ID", mid);
     char * bracketed = g_strdup_printf ("<%s>", mid);
-    g_mime_header_set (GMIME_OBJECT(msg)->headers, "Message-ID", bracketed);
+    g_mime_header_list_set (GMIME_OBJECT(msg)->headers, "Message-ID", bracketed);
     g_free (bracketed);
   }
 }
@@ -928,27 +979,27 @@ PostUI :: new_message_from_ui (Mode mode)
   // headers from the ui: To
   const StringView to (gtk_entry_get_text (GTK_ENTRY(_to_entry)));
   if (!to.empty())
-    g_mime_message_add_recipients_from_string (msg, (char*)GMIME_RECIPIENT_TYPE_TO, to.str);
+    pan_g_mime_message_add_recipients_from_string (msg, GMIME_RECIPIENT_TYPE_TO, to.str);
 
   // headers from the ui: Newsgroups
   const StringView groups (gtk_entry_get_text (GTK_ENTRY(_groups_entry)));
   if (!groups.empty())
-    g_mime_message_set_header (msg, "Newsgroups", groups.str);
+    g_mime_object_set_header ((GMimeObject *) msg, "Newsgroups", groups.str);
 
   // headers from the ui: Followup-To
   const StringView followupto (gtk_entry_get_text (GTK_ENTRY(_followupto_entry)));
   if (!followupto.empty())
-    g_mime_message_set_header (msg, "Followup-To", followupto.str);
+    g_mime_object_set_header ((GMimeObject *) msg, "Followup-To", followupto.str);
 
   // headers from the ui: Reply-To
   const StringView replyto (gtk_entry_get_text (GTK_ENTRY(_replyto_entry)));
   if (!replyto.empty())
-    g_mime_message_set_header (msg, "Reply-To", replyto.str);
+    g_mime_object_set_header ((GMimeObject *) msg, "Reply-To", replyto.str);
 
   // add the 'hidden headers'
   foreach_const (str2str_t, _hidden_headers, it)
     if ((mode==DRAFTING) || (it->first.find ("X-Draft-")!=0))
-      g_mime_message_set_header (msg, it->first.c_str(), it->second.c_str());
+      g_mime_object_set_header ((GMimeObject *) msg, it->first.c_str(), it->second.c_str());
 
   // build headers from the 'more headers' entry field
   std::map<std::string,std::string> headers;
@@ -964,14 +1015,14 @@ PostUI :: new_message_from_ui (Mode mode)
     val.trim ();
     std::string key_str (key.to_string());
     if (extra_header_is_editable (key, val))
-      g_mime_message_set_header (msg, key.to_string().c_str(),
-                                      val.to_string().c_str());
+      g_mime_object_set_header ((GMimeObject *) msg, key.to_string().c_str(),
+                                val.to_string().c_str());
   }
   g_free (pch);
 
   // User-Agent
   if (mode==POSTING && _prefs.get_flag (USER_AGENT_PREFS_KEY, true))
-    g_mime_message_set_header (msg, "User-Agent", get_user_agent());
+    g_mime_object_set_header ((GMimeObject *) msg, "User-Agent", get_user_agent());
 
   // Message-ID
   if (mode==POSTING && _prefs.get_flag (MESSAGE_ID_PREFS_KEY, false)) {
@@ -987,22 +1038,22 @@ PostUI :: new_message_from_ui (Mode mode)
   const std::string charset ((mode==POSTING && !_charset.empty()) ? _charset : "UTF-8");
   if (charset != "UTF-8") {
     // add a wrapper to convert from UTF-8 to $charset
-    GMimeStream * tmp = g_mime_stream_filter_new_with_stream (stream);
+    GMimeStream * tmp = g_mime_stream_filter_new (stream);
     g_object_unref (stream);
     GMimeFilter * filter = g_mime_filter_charset_new ("UTF-8", charset.c_str());
     g_mime_stream_filter_add (GMIME_STREAM_FILTER(tmp), filter);
     g_object_unref (filter);
     stream = tmp;
   }
-  GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_PART_ENCODING_DEFAULT);
+  GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_DEFAULT);
   g_object_unref (stream);
   GMimePart * part = g_mime_part_new ();
   pch = g_strdup_printf ("text/plain; charset=%s", charset.c_str());
   GMimeContentType * type = g_mime_content_type_new_from_string (pch);
   g_free (pch);
-  g_mime_part_set_content_type (part, type); // part owns type now. type isn't refcounted.
+  g_mime_object_set_content_type ((GMimeObject *) part, type); // part owns type now. type isn't refcounted.
   g_mime_part_set_content_object (part, content_object);
-  g_mime_part_set_encoding (part, GMIME_PART_ENCODING_8BIT);
+  g_mime_part_set_content_encoding (part, GMIME_CONTENT_ENCODING_8BIT);
   g_object_unref (content_object);
   g_mime_message_set_mime_part (msg, GMIME_OBJECT(part));
   g_object_unref (part);
@@ -1036,7 +1087,7 @@ PostUI :: save_draft ()
 
     errno = 0;
     std::ofstream o (filename);
-    char * pch = g_mime_message_to_string (msg);
+    char * pch = g_mime_object_to_string ((GMimeObject *) msg);
     o << pch;
     o.close ();
 
@@ -1083,7 +1134,7 @@ PostUI :: create_body_widget (GtkTextBuffer*& buf, GtkWidget*& view, const Prefs
   // always use a monospace font in the compose window
   const std::string str (prefs.get_string ("monospace-font", "Monospace 10"));
   PangoFontDescription *pfd (pango_font_description_from_string (str.c_str()));
-  gtk_widget_modify_font (view, pfd);
+  gtk_widget_override_font (view, pfd);
 
   // figure out how wide the text is before the wrap point
   PangoContext * context = gtk_widget_create_pango_context (view);
@@ -1125,8 +1176,8 @@ void
 PostUI :: update_profile_combobox ()
 {
   // get the old selection
-  GtkComboBox * combo (GTK_COMBO_BOX (_from_combo));
-  char * active_text = gtk_combo_box_get_active_text (combo);
+  GtkComboBoxText * combo (GTK_COMBO_BOX_TEXT (_from_combo));
+  char * active_text = gtk_combo_box_text_get_active_text (combo);
 
   // if there's not already a selection,
   // pull the default for the newsgroup
@@ -1145,9 +1196,9 @@ PostUI :: update_profile_combobox ()
   }
 
   // tear out the old entries
-  GtkTreeModel * model (gtk_combo_box_get_model (combo));
+  GtkTreeModel * model (gtk_combo_box_get_model (GTK_COMBO_BOX(combo)));
   for (int i(0), qty(gtk_tree_model_iter_n_children(model,0)); i<qty; ++i)
-    gtk_combo_box_remove_text (combo, 0);
+    gtk_combo_box_text_remove (combo, 0);
 
   // add the new entries
   typedef std::set<std::string> names_t;
@@ -1155,14 +1206,14 @@ PostUI :: update_profile_combobox ()
   int index (0);
   int sel_index (0);
   foreach_const (names_t, profile_names, it) {
-    gtk_combo_box_append_text (combo, it->c_str());
+    gtk_combo_box_text_append_text (combo, it->c_str());
     if (active_text && (*it == active_text))
       sel_index = index;
     ++index;
   }
  
   // ensure _something_ is selected...
-  gtk_combo_box_set_active (combo, sel_index);
+  gtk_combo_box_set_active (GTK_COMBO_BOX(combo), sel_index);
 
   // cleanup
   g_free (active_text);
@@ -1302,7 +1353,7 @@ Profile
 PostUI :: get_current_profile ()
 {
   Profile profile;
-  char * pch = gtk_combo_box_get_active_text (GTK_COMBO_BOX(_from_combo));
+  char * pch = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT(_from_combo));
   if (pch) {
     _profiles.get_profile (pch, profile);
     g_free (pch);
@@ -1490,6 +1541,16 @@ PostUI :: utf8ize (const StringView& in) const
   return content_to_utf8 (in, _charset.c_str(), local_charset);
 }
 
+#if GMIME_MINOR_VERSION == 4
+namespace {
+  inline GMimeStream* gmime_header_list_get_stream(GMimeHeaderList *hl)
+  {
+    // the name of this function was changed for 2.6
+    return gmime_header_list_has_raw(hl);
+  }
+}
+#endif
+
 void
 PostUI :: set_message (GMimeMessage * message)
 {
@@ -1504,16 +1565,16 @@ PostUI :: set_message (GMimeMessage * message)
   std::string s = utf8ize (g_mime_message_get_subject (message));
   gtk_entry_set_text (GTK_ENTRY(_subject_entry), s.c_str());
 
-  s = utf8ize (g_mime_message_get_header (message, "Newsgroups"));
+  s = utf8ize (g_mime_object_get_header ((GMimeObject *) message, "Newsgroups"));
   gtk_entry_set_text (GTK_ENTRY(_groups_entry), s.c_str());
 
-  s = utf8ize (g_mime_message_get_header (message, "Followup-To"));
+  s = utf8ize (g_mime_object_get_header ((GMimeObject *) message, "Followup-To"));
   gtk_entry_set_text (GTK_ENTRY(_followupto_entry), s.c_str());
 
-  s = utf8ize (g_mime_message_get_header (message, "Reply-To"));
+  s = utf8ize (g_mime_object_get_header ((GMimeObject *) message, "Reply-To"));
   gtk_entry_set_text (GTK_ENTRY(_replyto_entry), s.c_str());
 
-  const InternetAddressList * addresses = g_mime_message_get_recipients (message, GMIME_RECIPIENT_TYPE_TO);
+  InternetAddressList * addresses = g_mime_message_get_recipients (message, GMIME_RECIPIENT_TYPE_TO);
   char * pch  = internet_address_list_to_string (addresses, true);
   s = utf8ize (pch);
   gtk_entry_set_text (GTK_ENTRY(_to_entry), s.c_str());
@@ -1521,21 +1582,41 @@ PostUI :: set_message (GMimeMessage * message)
 
   // update 'other headers'
   SetMessageForeachHeaderData data;
-  if (message->mime_part && g_mime_header_has_raw (message->mime_part->headers))
-    g_mime_header_foreach (message->mime_part->headers, set_message_foreach_header_func, &data);
-  g_mime_header_foreach (GMIME_OBJECT(message)->headers, set_message_foreach_header_func, &data);
+  const char *name, *value;
+  GMimeHeaderIter iter;
+
+  if (message->mime_part && g_mime_header_list_get_stream (message->mime_part->headers)) {
+    if (g_mime_header_list_get_iter (message->mime_part->headers, &iter)) {
+      do {
+        value = g_mime_header_iter_get_value (&iter);
+        name = g_mime_header_iter_get_name (&iter);
+        set_message_foreach_header_func (name, value, &data);
+      } while (g_mime_header_iter_next (&iter));
+    }
+  }
+
+  if (g_mime_header_list_get_iter (GMIME_OBJECT (message)->headers, &iter)) {
+    do {
+      value = g_mime_header_iter_get_value (&iter);
+      name = g_mime_header_iter_get_name (&iter);
+      set_message_foreach_header_func (name, value, &data);
+    } while (g_mime_header_iter_next (&iter));
+  }
+
   s = utf8ize (data.visible_headers);
   gtk_text_buffer_set_text (_headers_buf, s.c_str(), -1);
   _hidden_headers = data.hidden_headers;
 
   // update body
   int ignored;
-  char * tmp = g_mime_message_get_body (message, true, &ignored);
+  char * tmp = pan_g_mime_message_get_body (message, &ignored);
   s = utf8ize (tmp);
   g_free (tmp);
   if (!s.empty()) {
-    s = TextMassager().fill (s);
-    s += "\n\n";
+    if (_prefs.get_flag ("compose-wrap-enabled", true)) {
+      s = TextMassager().fill (s);
+      s += "\n\n";
+    }
     gtk_text_buffer_set_text (_body_buf, s.c_str(), s.size());
   }
 
@@ -1673,7 +1754,7 @@ PostUI :: create_main_tab ()
   gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
   gtk_table_attach (GTK_TABLE(t), l, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
 
-  w = _from_combo = gtk_combo_box_new_text ();
+  w = _from_combo = gtk_combo_box_text_new ();
   gtk_cell_layout_clear (GTK_CELL_LAYOUT(w));
   GtkCellRenderer * r =  gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT(w), r, true);
@@ -1731,7 +1812,7 @@ PostUI :: create_main_tab ()
   GtkWidget * v = gtk_vbox_new (false, PAD);
   gtk_container_set_border_width (GTK_CONTAINER(v), PAD);
   gtk_box_pack_start (GTK_BOX(v), t, false, false, 0);
-  gtk_box_pack_start_defaults (GTK_BOX(v), w);
+  pan_box_pack_start_defaults (GTK_BOX(v), w);
   return v;
 }
 
@@ -1770,7 +1851,7 @@ PostUI :: create_extras_tab ()
   w = _followupto_entry = gtk_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
   /* i18n: "poster" is a key used by many newsreaders.  probably safest to keep this key in english. */
-  gtk_tooltips_set_tip (GTK_TOOLTIPS(_ttips), w, _("The newsgroups where replies to your message should go.  This is only needed if it differs from the \"Newsgroups\" header.\n\nTo direct all replies to your email address, use \"Followup-To: poster\""), 0);
+gtk_widget_set_tooltip_text (w, _("The newsgroups where replies to your message should go.  This is only needed if it differs from the \"Newsgroups\" header.\n\nTo direct all replies to your email address, use \"Followup-To: poster\""));
   gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, fe, fill, 0, 0);
 
   //  Reply-To
@@ -1784,7 +1865,7 @@ PostUI :: create_extras_tab ()
 
   w = _replyto_entry = gtk_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-  gtk_tooltips_set_tip (GTK_TOOLTIPS(_ttips), w, _("The email account where mail replies to your posted message should go.  This is only needed if it differs from the \"From\" header."), 0);
+gtk_widget_set_tooltip_text (w, _("The email account where mail replies to your posted message should go.  This is only needed if it differs from the \"From\" header."));
   gtk_table_attach (GTK_TABLE(t), w, 1, 2, row, row+1, fe, fill, 0, 0);
 
   //  Extra Headers
@@ -1874,22 +1955,23 @@ PostUI :: PostUI (GtkWindow    * parent,
   g_assert (profiles.has_profiles());
   g_return_if_fail (message != 0);
 
+  ua_extra = prefs.get_flag(USER_AGENT_EXTRA_PREFS_KEY, false);
+
   // create the window
   _root = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   g_signal_connect (_root, "delete-event", G_CALLBACK(delete_event_cb), this);
   gtk_window_set_role (GTK_WINDOW(_root), "pan-post-window");
   gtk_window_set_title (GTK_WINDOW(_root), _("Post Article"));
-  gtk_window_set_default_size (GTK_WINDOW(_root), -1, 450);
+  int w,h;
+  w = _prefs.get_int("post-ui-width", -1);
+  h = _prefs.get_int("post-ui-height", 450);
+  gtk_window_set_default_size (GTK_WINDOW(_root), w, h);
   g_object_set_data_full (G_OBJECT(_root), "post-ui", this, delete_post_ui);
   if (parent) {
     gtk_window_set_transient_for (GTK_WINDOW(_root), parent);
     gtk_window_set_position (GTK_WINDOW(_root), GTK_WIN_POS_CENTER_ON_PARENT);
   }
 
-  _ttips = gtk_tooltips_new ();
-  g_object_ref_sink_pan (G_OBJECT(_ttips));
-  g_object_weak_ref (G_OBJECT(_root), (GWeakNotify)g_object_unref, _ttips);
-
   // populate the window
   GtkWidget * vbox = gtk_vbox_new (false, PAD_SMALL);
   GtkWidget * menu_vbox = gtk_vbox_new (false, PAD_SMALL);
@@ -1902,7 +1984,7 @@ PostUI :: PostUI (GtkWindow    * parent,
   GtkWidget * notebook = gtk_notebook_new ();
   gtk_notebook_append_page (GTK_NOTEBOOK(notebook), create_main_tab(), gtk_label_new_with_mnemonic(_("_Message")));
   gtk_notebook_append_page (GTK_NOTEBOOK(notebook), create_extras_tab(), gtk_label_new_with_mnemonic(_("More _Headers")));
-  gtk_box_pack_start_defaults (GTK_BOX(vbox), notebook);
+  pan_box_pack_start_defaults (GTK_BOX(vbox), notebook);
 
   // remember this message, but don't put it in the text view yet.
   // we have to wait for it to be realized first so that wrapping
