@@ -51,6 +51,7 @@ namespace pan
     public:
       GtkWidget * root() { return _root; }
       void rot13_selection ();
+      void wrap_selection ();
       void wrap_body ();
       void spawn_editor ();
       void manage_profiles ();
@@ -111,7 +112,6 @@ namespace pan
       str2str_t _profile_headers;
       std::string _unchanged_body;
       int _wrap_pixels;
-      GtkTooltips * _ttips;
 
     private:
       void add_actions (GtkWidget* box);
@@ -143,6 +143,7 @@ namespace pan
 
     public:
       void set_spellcheck_enabled (bool);
+	  void spawn_editor_dead(char *);
 
   };
 }
