@@ -31,6 +31,7 @@ extern "C" {
 #include "prefs-ui.h"
 #include "tango-colors.h"
 #include "url.h"
+#include "gtk_compat.h"
 
 using namespace pan;
 
@@ -41,19 +42,6 @@ namespace
     delete static_cast<PrefsDialog*>(castme);
   }
 
-#if !GTK_CHECK_VERSION(2,6,0)
-  char* gtk_combo_box_get_active_text (GtkComboBox * combo_box)
-  {
-    char * text (0);
-    GtkTreeIter  iter;
-    if (gtk_combo_box_get_active_iter (combo_box, &iter)) {
-      GtkTreeModel * model = gtk_combo_box_get_model (combo_box);
-      gtk_tree_model_get (model, &iter, 0, &text, -1);
-    }
-    return text;
-  }
-#endif
-
   void response_cb (GtkDialog * dialog, int, gpointer)
   {
     gtk_widget_destroy (GTK_WIDGET(dialog));
@@ -102,6 +90,22 @@ namespace
     return r;
   }
 
+  void spin_value_changed_cb( GtkSpinButton *spin, gpointer data)
+  {
+    const char * key = (const char*) g_object_get_data (G_OBJECT(spin), PREFS_KEY);
+    Prefs *prefs = static_cast<Prefs*>(data);
+    prefs->set_int(key, gtk_spin_button_get_value_as_int(spin));
+  }
+  GtkWidget* new_spin_button (const char *key, int low, int high, Prefs &prefs)
+  {
+    guint tm = prefs.get_int(key, 5 );
+    GtkAdjustment *adj = (GtkAdjustment*) gtk_adjustment_new(tm, low, high, 1.0, 1.0, 0.0);
+    GtkWidget *w = gtk_spin_button_new( adj, 1.0, 0);
+    g_object_set_data_full(G_OBJECT(w), PREFS_KEY, g_strdup(key), g_free);
+    g_signal_connect (w, "value_changed", G_CALLBACK(spin_value_changed_cb), &prefs);
+    return w;
+  }
+
   GtkWidget* new_orient_radio (GtkWidget* prev, const char* label, const char* value, std::string& cur, Prefs& prefs)
   {
     GtkWidget * r = prev==0
@@ -144,10 +148,10 @@ namespace
     }
   }
 
-  void set_prefs_string_from_combo_box_entry (GtkComboBox * c, gpointer user_data)
+  void set_prefs_string_from_combo_box_entry (GtkComboBoxText * c, gpointer user_data)
   {
     const char * key = (const char*) g_object_get_data (G_OBJECT(c), PREFS_KEY);
-    char * val = gtk_combo_box_get_active_text (c);
+    char * val = gtk_combo_box_text_get_active_text (c);
     static_cast<Prefs*>(user_data)->set_string (key, val);
     g_free (val);
   }
@@ -159,10 +163,10 @@ namespace
     const char * key = "editor";
     const std::string editor = prefs.get_string (key, *editors.begin());
     editors.insert (editor);
-    GtkWidget * c = gtk_combo_box_entry_new_text ();
+    GtkWidget * c = gtk_combo_box_text_new_with_entry ();
     g_object_set_data_full (G_OBJECT(c), PREFS_KEY, g_strdup(key), g_free);
     foreach_const (std::set<std::string>, editors, it)
-      gtk_combo_box_append_text (GTK_COMBO_BOX(c), it->c_str());
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(c), it->c_str());
     gtk_combo_box_set_active (GTK_COMBO_BOX(c),
                               (int)std::distance (editors.begin(), editors.find(editor)));
     g_signal_connect (c, "changed", G_CALLBACK(set_prefs_string_from_combo_box_entry), &prefs);
@@ -462,13 +466,18 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_wide_control (t, &row, w);
   HIG::workarea_add_section_divider (t, &row);
   HIG :: workarea_add_section_title (t, &row, _("Articles"));
-    HIG :: workarea_add_section_spacer (t, row, 3);
+    HIG :: workarea_add_section_spacer (t, row, 4);
     w = new_check_button (_("Space selects next article rather than next unread"), "space-selects-next-article", true, prefs);
     HIG :: workarea_add_wide_control (t, &row, w);
     w = new_check_button (_("Smooth scrolling"), "smooth-scrolling", true, prefs);
     HIG :: workarea_add_wide_control (t, &row, w);
     w = new_check_button (_("Clear article cache on shutdown"), "clear-article-cache-on-shutdown", false, prefs);
     HIG :: workarea_add_wide_control (t, &row, w);
+    w = new_spin_button ("newsrc-autosave-timeout-min", 0, 60, prefs);
+    l = gtk_label_new(_("Minutes to autosave newsrc files."));
+    gtk_misc_set_alignment (GTK_MISC(l), 0.0, 0.5);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(l), w);
+    HIG::workarea_add_row (t, &row, w, l);
   HIG :: workarea_finish (t, &row);
   gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, gtk_label_new_with_mnemonic(_("_Behavior")));
 
@@ -546,42 +555,42 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
   HIG :: workarea_add_section_title (t, &row, _("Header Pane"));
     HIG :: workarea_add_section_spacer(t, row, 5);
     h = gtk_hbox_new (false, PAD);
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-watched-fg", "black", prefs));
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-watched-bg", TANGO_CHAMELEON_LIGHT, prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-watched-fg", "black", prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-watched-bg", TANGO_CHAMELEON_LIGHT, prefs));
     HIG :: workarea_add_row (t, &row, _("Scores of 9999 or more:"), h);
     h = gtk_hbox_new (false, PAD);
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-high-fg", "black", prefs));
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-high-bg", TANGO_BUTTER_LIGHT, prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-high-fg", "black", prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-high-bg", TANGO_BUTTER_LIGHT, prefs));
     HIG :: workarea_add_row (t, &row, _("Scores from 5000 to 9998:"), h);
     h = gtk_hbox_new (false, PAD);
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-medium-fg", "black", prefs));
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-medium-bg", TANGO_SKY_BLUE_LIGHT, prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-medium-fg", "black", prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-medium-bg", TANGO_SKY_BLUE_LIGHT, prefs));
     HIG :: workarea_add_row (t, &row, _("Scores from 1 to 4999:"), h);
     h = gtk_hbox_new (false, PAD);
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-low-fg", TANGO_ALUMINUM_2, prefs));
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-low-bg", "black", prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-low-fg", TANGO_ALUMINUM_2, prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-low-bg", "black", prefs));
     HIG :: workarea_add_row (t, &row, _("Scores from -9998 to -1:"), h);
     h = gtk_hbox_new (false, PAD);
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-ignored-fg", TANGO_ALUMINUM_4, prefs));
-    gtk_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-ignored-bg", "black", prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Text:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-ignored-fg", TANGO_ALUMINUM_4, prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), gtk_label_new (_("Background:")));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("score-color-ignored-bg", "black", prefs));
     HIG :: workarea_add_row (t, &row, _("Scores of -9999 or less:"), h);
   HIG :: workarea_add_section_divider (t, &row);
   HIG :: workarea_add_section_title (t, &row, _("Body Pane"));
     HIG :: workarea_add_section_spacer (t, row, 3);
     h = gtk_hbox_new (false, PAD_SMALL);
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("body-pane-color-quote-1", TANGO_CHAMELEON_DARK, prefs));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("body-pane-color-quote-2", TANGO_ORANGE_DARK, prefs));
-    gtk_box_pack_start_defaults (GTK_BOX(h), new_color_button ("body-pane-color-quote-3", TANGO_PLUM_DARK, prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("body-pane-color-quote-1", TANGO_CHAMELEON_DARK, prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("body-pane-color-quote-2", TANGO_ORANGE_DARK, prefs));
+    pan_box_pack_start_defaults (GTK_BOX(h), new_color_button ("body-pane-color-quote-3", TANGO_PLUM_DARK, prefs));
     HIG :: workarea_add_row (t, &row, _("Quoted Text:"), h);
     HIG :: workarea_add_row (t, &row, _("URL:"), new_color_button ("body-pane-color-url", TANGO_SKY_BLUE_DARK, prefs));
     HIG :: workarea_add_row (t, &row, _("Signature:"), new_color_button ("body-pane-color-signature", TANGO_SKY_BLUE_LIGHT, prefs));
@@ -604,6 +613,6 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
   gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, gtk_label_new_with_mnemonic(_("A_pplications")));
 
   gtk_widget_show_all (notebook);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), notebook, true, true, 0);
+  gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area( GTK_DIALOG(dialog))), notebook, true, true, 0);
   _root = dialog;
 }
