@@ -20,17 +20,19 @@
 #include <config.h>
 #include <cstdlib>
 #include <cstring>
+#include <climits>
 extern "C" {
   #include <glib.h>
   #include <glib/gi18n.h>
   #include <gtk/gtk.h>
 }
-#include <pan/general/foreach.h>
+#include <pan/general/macros.h>
 #include <pan/general/quark.h>
 #include <pan/data/data.h>
 #include "server-ui.h"
 #include "pad.h"
 #include "hig.h"
+#include "gtk_compat.h"
 
 using namespace pan;
 
@@ -81,7 +83,7 @@ namespace
   }
 
   void
-  edit_dialog_populate (Data& data, const Quark& server, ServerEditDialog * d)
+  edit_dialog_populate (Data&, const Quark& server, ServerEditDialog * d)
   {
     // sanity clause
     g_return_if_fail (d!=0);
@@ -215,24 +217,19 @@ pan :: server_edit_dialog_new (Data& data, Queue& queue, GtkWindow * window, con
   ***  workarea
   **/
 
-  GtkTooltips * ttips = gtk_tooltips_new ();
-  g_object_ref_sink_pan (G_OBJECT(ttips));
-  g_object_weak_ref (G_OBJECT(dialog), (GWeakNotify)g_object_unref, ttips);
-
-
   int row (0);
   GtkWidget * t (HIG::workarea_create ());
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d->dialog)->vbox), t, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX( gtk_dialog_get_content_area( GTK_DIALOG(d->dialog))), t, TRUE, TRUE, 0);
   HIG::workarea_add_section_title (t, &row, _("Location"));
     HIG::workarea_add_section_spacer (t, row, 2);
 
     GtkWidget * w = d->address_entry = gtk_entry_new ();
-    gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The news server's actual address, e.g. \"news.mynewsserver.com\""), NULL);
+    gtk_widget_set_tooltip_text( w, _("The news server's actual address, e.g. \"news.mynewsserver.com\""));
     HIG::workarea_add_row (t, &row, _("_Address:"), w, NULL);
 
-    GtkAdjustment * a = GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 1.0, ULONG_MAX, 1.0, 1.0, 1.0));
+    GtkAdjustment * a = GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 1.0, ULONG_MAX, 1.0, 1.0, 0.0));
     w = d->port_spin = gtk_spin_button_new (GTK_ADJUSTMENT(a), 1.0, 0u);
-    gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The news server's port number.  Typically 119."), NULL);
+    gtk_widget_set_tooltip_text( w, _("The news server's port number.  Typically 119."));
     HIG::workarea_add_row (t, &row, _("Por_t:"), w, NULL);
 
   HIG::workarea_add_section_divider (t, &row);
@@ -241,19 +238,19 @@ pan :: server_edit_dialog_new (Data& data, Queue& queue, GtkWindow * window, con
 
     w = d->auth_username_entry = gtk_entry_new ();
     HIG::workarea_add_row (t, &row, _("_Username:"), w, NULL);
-    gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The username to give the server when asked.  If your server doesn't require authentication, you can leave this blank."), NULL);
+    gtk_widget_set_tooltip_text( w, _("The username to give the server when asked.  If your server doesn't require authentication, you can leave this blank."));
 
     w = d->auth_password_entry = gtk_entry_new ();
     gtk_entry_set_visibility (GTK_ENTRY(w), FALSE);
     HIG::workarea_add_row (t, &row, _("_Password:"), w, NULL);
-    gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The password to give the server when asked.  If your server doesn't require authentication, you can leave this blank."), NULL);
+    gtk_widget_set_tooltip_text( w, _("The password to give the server when asked.  If your server doesn't require authentication, you can leave this blank."));
 
   HIG::workarea_add_section_divider (t, &row);
   HIG::workarea_add_section_title (t, &row, _("Settings"));
     HIG::workarea_add_section_spacer (t, row, 2);
 
     const int DEFAULT_MAX_PER_SERVER (4);
-    a = GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 0.0, DEFAULT_MAX_PER_SERVER, 1.0, 1.0, 1.0));
+    a = GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 0.0, DEFAULT_MAX_PER_SERVER, 1.0, 1.0, 0.0));
     d->connection_limit_spin = w = gtk_spin_button_new (GTK_ADJUSTMENT(a), 1.0, 0u);
     HIG::workarea_add_row (t, &row, _("Connection _Limit:"), w, NULL);
 
@@ -299,7 +296,7 @@ pan :: server_edit_dialog_new (Data& data, Queue& queue, GtkWindow * window, con
     GtkWidget * e = gtk_event_box_new ();
     gtk_container_add (GTK_CONTAINER(e), l);
     gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-    gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), e, _("Fallback servers are used for articles that can't be found on the primaries.  One common approach is to use free servers as primaries and subscription servers as fallbacks."), NULL);
+    gtk_widget_set_tooltip_text( e, _("Fallback servers are used for articles that can't be found on the primaries.  One common approach is to use free servers as primaries and subscription servers as fallbacks."));
     HIG::workarea_add_row (t, &row, e, w);
 
   d->server = server;
@@ -378,9 +375,7 @@ namespace
     foreach_const (quarks_t, servers, it)
     {
       const Quark& server (*it);
-      int port;
-      std::string addr;
-      d->data.get_server_addr (*it, addr, port);
+      std::string addr(d->data.get_server_address (*it));
 
       GtkTreeIter iter;
       gtk_list_store_append (d->servers_store, &iter);
@@ -402,7 +397,7 @@ namespace
   }
 
   void
-  server_list_dialog_response_cb (GtkDialog * dialog, int response, gpointer user_data)
+  server_list_dialog_response_cb (GtkDialog * dialog, int, gpointer)
   {
     gtk_widget_destroy (GTK_WIDGET(dialog));
   }
@@ -440,7 +435,7 @@ namespace
   }
 
   void
-  server_edit_dialog_destroy_cb (GtkWidget * w, gpointer user_data)
+  server_edit_dialog_destroy_cb (GtkWidget *, gpointer user_data)
   {
     if (GTK_IS_WIDGET (user_data))
     {
@@ -450,7 +445,7 @@ namespace
   }
 
   void
-  add_button_clicked_cb (GtkButton * button, gpointer user_data)
+  add_button_clicked_cb (GtkButton *, gpointer user_data)
   {
     const Quark empty_quark;
     GtkWidget * list_dialog = GTK_WIDGET (user_data);
@@ -461,7 +456,7 @@ namespace
   }
 
   void
-  edit_button_clicked_cb (GtkButton * button, gpointer user_data)
+  edit_button_clicked_cb (GtkButton *, gpointer user_data)
   {
     GtkWidget * list_dialog = GTK_WIDGET (user_data);
     ServerListDialog * d = (ServerListDialog*) g_object_get_data (G_OBJECT(list_dialog), "dialog");
@@ -474,13 +469,13 @@ namespace
   }
 
   void
-  server_tree_view_row_activated_cb (GtkTreeView *view, GtkTreePath *path, GtkTreeViewColumn *col, gpointer user_data)
+  server_tree_view_row_activated_cb (GtkTreeView*, GtkTreePath*, GtkTreeViewColumn*, gpointer user_data)
   {
     edit_button_clicked_cb (NULL, user_data);	
   }
 
   void
-  server_tree_view_selection_changed_cb (GtkTreeSelection * selection, gpointer user_data)
+  server_tree_view_selection_changed_cb (GtkTreeSelection*, gpointer user_data)
   {
     button_refresh ((ServerListDialog*)user_data);
   }
@@ -500,18 +495,14 @@ pan :: server_list_dialog_new (Data& data, Queue& queue, GtkWindow* parent)
                                                            NULL);
   g_free (title);
   gtk_window_set_role (GTK_WINDOW(w), "pan-servers-dialog");
-  gtk_window_set_policy (GTK_WINDOW(w), TRUE, TRUE, TRUE);
+  gtk_window_set_resizable (GTK_WINDOW(w), TRUE);
   g_signal_connect (GTK_OBJECT(w), "response", G_CALLBACK(server_list_dialog_response_cb), d);
   g_object_set_data_full (G_OBJECT(w), "dialog", d, delete_server_list_dialog);
 
-  GtkTooltips * tips = gtk_tooltips_new ();
-  g_object_ref_sink_pan (G_OBJECT(tips));
-  g_object_weak_ref (G_OBJECT(w), (GWeakNotify)g_object_unref, tips);
-
   // workarea
   GtkWidget * hbox = gtk_hbox_new (FALSE, PAD);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 12);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(w)->vbox), hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX( gtk_dialog_get_content_area( GTK_DIALOG(w))), hbox, TRUE, TRUE, 0);
 
 
   // create the list
@@ -544,24 +535,20 @@ pan :: server_list_dialog_new (Data& data, Queue& queue, GtkWindow* parent)
   // add button
   w = gtk_button_new_from_stock (GTK_STOCK_ADD);
   gtk_box_pack_start (GTK_BOX (bbox), w, FALSE, FALSE, 0);
-  gtk_tooltips_set_tip (tips, w, _("Add a Server"), NULL);
+  gtk_widget_set_tooltip_text(w, _("Add a Server"));
   g_signal_connect (w, "clicked", G_CALLBACK(add_button_clicked_cb), d->dialog);
 
   // edit button
-#if GTK_CHECK_VERSION(2,6,0)
   w = gtk_button_new_from_stock (GTK_STOCK_EDIT);
-#else
-  w = gtk_button_new_from_stock (GTK_STOCK_OPEN);
-#endif
   gtk_box_pack_start (GTK_BOX (bbox), w, FALSE, FALSE, 0);
-  gtk_tooltips_set_tip (tips, w, _("Edit a Server's Settings"), NULL);
+  gtk_widget_set_tooltip_text(w, _("Edit a Server's Settings"));
   g_signal_connect (w, "clicked", G_CALLBACK(edit_button_clicked_cb), d->dialog);
   d->edit_button = w;
 
   // remove button
   w = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
   gtk_box_pack_start (GTK_BOX (bbox), w, FALSE, FALSE, 0);
-  gtk_tooltips_set_tip (tips, w, _("Remove a Server"), NULL);
+  gtk_widget_set_tooltip_text(w, _("Remove a Server"));
   g_signal_connect (w, "clicked", G_CALLBACK(remove_button_clicked_cb), d);
   d->remove_button = w;
 
