@@ -27,8 +27,8 @@ extern "C" {
 }
 #include <pan/general/debug.h>
 #include <pan/general/file-util.h>
-#include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/macros.h>
 #include <pan/general/quark.h>
 #include <pan/general/string-view.h>
 #include <pan/general/utf8-utils.h>
@@ -72,12 +72,12 @@ namespace
   };
 
   // called for open tags <foo bar='baz'>
-  void start_element (GMarkupParseContext *context,
+  void start_element (GMarkupParseContext *context         UNUSED,
                       const gchar         *element_name,
                       const gchar        **attribute_names,
                       const gchar        **attribute_vals,
                       gpointer             user_data,
-                      GError             **error)
+                      GError             **error           UNUSED)
   {
     MyContext& mc (*static_cast<MyContext*>(user_data));
 
@@ -101,10 +101,10 @@ namespace
   }
 
   // Called for close tags </foo>
-  void end_element    (GMarkupParseContext *context,
+  void end_element    (GMarkupParseContext *context       UNUSED,
                        const gchar         *element_name,
                        gpointer             user_data,
-                       GError             **error)
+                       GError             **error         UNUSED)
   {
     MyContext& mc (*static_cast<MyContext*>(user_data));
 
@@ -139,11 +139,11 @@ namespace
     }
   }
 
-  void text (GMarkupParseContext *context,
+  void text (GMarkupParseContext *context    UNUSED,
              const gchar         *text,
              gsize                text_len,
              gpointer             user_data,
-             GError             **error)
+             GError             **error      UNUSED)
   {
     static_cast<MyContext*>(user_data)->text.assign (text, text_len);
   }
@@ -215,6 +215,7 @@ namespace
   }
 }
 
+/* Saves all current tasks to tasks.nzb */
 std::ostream&
 NZB :: nzb_to_xml (std::ostream             & out,
                    const std::vector<Task*> & tasks)
@@ -286,3 +287,65 @@ NZB :: nzb_to_xml (std::ostream             & out,
   out << indent(--depth) << "</nzb>\n";
   return out;
 }
+
+/* Saves selected files to a chosen XML file */
+std::ostream&
+NZB :: nzb_to_xml_file (std::ostream             & out,
+                   const std::vector<Task*> & tasks)
+{
+  int depth (0);
+
+  out << "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"
+      << "<!DOCTYPE nzb PUBLIC \"-//newzBin//DTD NZB 1.0//EN\" \"http://www.newzbin.com/DTD/nzb/nzb-1.0.dtd\">\n"
+      << indent(depth++)
+      << "<nzb xmlns=\"http://www.newzbin.com/DTD/2003/nzb\">\n";
+
+  foreach_const (tasks_t, tasks, it)
+  {
+    TaskArticle * task (dynamic_cast<TaskArticle*>(*it));
+    if (!task) // not a download task...
+      continue;
+
+    const Article& a (task->get_article());
+    out << indent(depth++)
+        << "<file" << " poster=\"";
+    escaped (out, a.author.to_view());
+    out  << "\" date=\"" << a.time_posted << "\" subject=\"";
+    escaped (out, a.subject.to_view()) << "\">\n";
+
+    // what groups was this crossposted in?
+    quarks_t groups;
+    foreach_const (Xref, a.xref, xit)
+      groups.insert (xit->group);
+    out << indent(depth++) << "<groups>\n";
+    foreach_const (quarks_t, groups, git)
+      out << indent(depth) << "<group>" << *git << "</group>\n";
+    out << indent(--depth) << "</groups>\n";
+
+    // now for the parts...
+    out << indent(depth++) << "<segments>\n";
+    for (Article::part_iterator it(a.pbegin()), end(a.pend()); it!=end; ++it)
+    {
+      std::string mid = it.mid ();
+
+      // remove the surrounding < > as per nzb spec
+      if (mid.size()>=2 && mid[0]=='<') {
+        mid.erase (0, 1);
+        mid.resize (mid.size()-1);
+      }
+
+      // serialize this part
+      out << indent(depth)
+          << "<segment" << " bytes=\"" << it.bytes() << '"'
+                        << " number=\"" << it.number() << '"'
+                        << ">";
+      escaped(out, mid);
+      out  << "</segment>\n";
+    }
+    out << indent(--depth) << "</segments>\n";
+    out << indent(--depth) << "</file>\n";
+  }
+
+  out << indent(--depth) << "</nzb>\n";
+  return out;
+}
