@@ -19,12 +19,15 @@
 
 #include <config.h>
 #include <vector>
+#include <map>
+#include <cstring>
+#include <glib.h>
 extern "C" {
 #include <glib/gi18n.h>
 }
 #include <glib/gunicode.h>
 #include "text-massager.h"
-
+#include <pan/general/log.h>
 using namespace pan;
 
 TextMassager :: TextMassager ():
@@ -102,12 +105,128 @@ namespace
             content (c, clen) { }
    };
 
-
    typedef std::vector<Paragraph> paragraphs_t;
    typedef paragraphs_t::iterator p_it;
 
+   void merge_fixed (paragraphs_t &paragraphs, lines_t &lines, int wrap_col)
+   {
+     std::map<std::string, int> max_map;
+     std::map<std::string, int>::iterator map_end = max_map.end();
+     int prev_content_len = 0;
+     StringView cur_leader;
+     std::string cur_content;
+     bool prev_ends_with_punct = true;
+
+     for (lines_cit it=lines.begin(), end=lines.end(); it!=end; ++it)
+     {
+       const Line& line (*it);
+       int line_len = line.leader.len + line.content.len;
+       if (map_end != max_map.find(line.leader))
+         max_map[line.leader] = MAX(max_map[line.leader], line_len );
+       else
+         // use 12 as the minimum line length
+         max_map.insert( std::make_pair(line.leader, MAX(line_len, 12) ) );
+     }
+
+     for (lines_cit it=lines.begin(), end=lines.end(); it!=end; ++it)
+     {
+        const Line& line (*it);
+        bool paragraph_end = true;
+        bool hard_break = false;
+
+        if (cur_content.empty() || line.leader==cur_leader)
+           paragraph_end = false;
+
+        if (line.content.empty()) {
+           hard_break = prev_content_len!=0;
+           paragraph_end = true;
+        }
+
+        // if first word could have been wrapped onto previous line
+        // line but wasn't assume deliberate line break.
+        if (!paragraph_end && prev_content_len && line.content.len)
+        {
+          unsigned int space = max_map[line.leader] - (prev_content_len + line.leader.len) - 1;
+          if ( space > 0 && ((line.content.len < space)
+                              || g_utf8_strchr (line.content.str, space, ' ')) )
+            //paragraph_end = true;
+            paragraph_end = prev_ends_with_punct;
+        }
+
+        if (paragraph_end) // the new line is a new paragraph, so save old
+        {
+           paragraphs.push_back (Paragraph (
+              cur_leader.str, cur_leader.len,
+              cur_content.c_str(), cur_content.size()));
+           cur_leader = line.leader;
+           cur_content = line.content.to_string();
+           if (hard_break) {
+              paragraphs.push_back (Paragraph (
+                 cur_leader.str, cur_leader.len, "", 0));
+            }
+        }
+        else // append to the content
+        {
+           if (!cur_content.empty())
+              cur_content += ' ';
+           cur_leader = line.leader;
+           cur_content.insert (cur_content.end(),
+                               line.content.begin(),
+                               line.content.end());
+        }
+
+        prev_content_len = line.content.len;
+        if ( prev_content_len > 1)
+          prev_ends_with_punct = g_unichar_ispunct ( g_utf8_get_char (
+            g_utf8_find_prev_char ( line.content.str, line.content.str + line.content.len )));
+        else
+          prev_ends_with_punct = true;
+     }
+   }
+
+  void merge_flowed (paragraphs_t &paragraphs, lines_t &lines)
+  {
+    StringView cur_leader;
+    std::string cur_content;
+    bool prev_flowed=true;
+
+    for (lines_cit it=lines.begin(), end=lines.end(); it!=end; ++it)
+    {
+      const Line& line (*it);
+
+      if (line.leader != cur_leader)
+        prev_flowed = false;
+
+      if (!prev_flowed) // the new line is a new paragraph, so save old
+      {
+         paragraphs.push_back (Paragraph (
+            cur_leader.str, cur_leader.len,
+            cur_content.c_str(), cur_content.size()));
+         cur_leader = line.leader;
+         cur_content = line.content.to_string();
+      }
+      else // append to the content
+      {
+         if (!cur_content.empty())
+            cur_content += ' ';
+         cur_leader = line.leader;
+         cur_content.insert (cur_content.end(),
+                             line.content.begin(),
+                             line.content.end());
+      }
+      // flowed text ends with a space
+      if (line.content.len && line.content.str[line.content.len] == ' ')
+         prev_flowed = true;
+      else
+        prev_flowed = false;
+    }
+    //add final content
+    paragraphs.push_back (Paragraph (cur_leader.str, cur_leader.len,
+                            cur_content.c_str(), cur_content.size()));
+ }
+
    std::vector<Paragraph>
-   get_paragraphs (const TextMassager& tm, const StringView& body)
+   get_paragraphs (const TextMassager& tm, const StringView& body, bool flowed)
    {
       StringView mybody (body);
       StringView line;
@@ -139,52 +258,10 @@ namespace
       std::vector<Paragraph> paragraphs;
       if (!lines.empty())
       {
-         int prev_content_len = 0;
-         StringView cur_leader;
-         std::string cur_content;
-
-         for (lines_cit it=lines.begin(), end=lines.end(); it!=end; ++it)
-         {
-            const Line& line (*it);
-            bool paragraph_end = true;
-            bool hard_break = false;
-
-            if (cur_content.empty() || line.leader==cur_leader)
-               paragraph_end = false;
-
-            if (line.content.empty()) {
-               hard_break = prev_content_len!=0;
-               paragraph_end = true;
-            }
-
-            // we usually don't want to wrap really short lines
-            if (prev_content_len && prev_content_len<(tm.get_wrap_column()/2))
-               paragraph_end = true;
-
-            if (paragraph_end) // the new line is a new paragraph, so save old
-            {
-               paragraphs.push_back (Paragraph (
-                  cur_leader.str, cur_leader.len,
-                  cur_content.c_str(), cur_content.size()));
-               cur_leader = line.leader;
-               cur_content = line.content.to_string();
-               if (hard_break) {
-                  paragraphs.push_back (Paragraph (
-                     cur_leader.str, cur_leader.len, "", 0));
-                }
-            }
-            else // append to the content
-            {
-               if (!cur_content.empty())
-                  cur_content += ' ';
-               cur_leader = line.leader;
-               cur_content.insert (cur_content.end(),
-                                   line.content.begin(),
-                                   line.content.end());
-            }
-
-            prev_content_len = line.content.len;
-         }
+        if (flowed)
+          merge_flowed(paragraphs, lines);
+        else
+          merge_fixed(paragraphs, lines, tm.get_wrap_column());
 
         // Remember that empty line we added back up at the top?
         // We remove it now
@@ -201,21 +278,33 @@ namespace
                         int       column)
    {
       int pos = 0;
+      int space_len;
       char * linefeed_here = NULL;
 
       // walk through the entire string
       for (char *pch=str, *end=pch+len; pch!=end; )
       {
          // a linefeed could go here; remember this space
-         if (g_unichar_isspace (g_utf8_get_char (pch)) || *pch=='\n')
-            linefeed_here = pch;
+         gunichar ch = g_utf8_get_char (pch);
+         if (g_unichar_isspace ( ch ) || *pch=='\n')
+           if (g_unichar_break_type(ch) != G_UNICODE_BREAK_NON_BREAKING_GLUE)
+           {
+             linefeed_here = pch;
+             // not all spaces are single char
+             space_len = g_utf8_next_char (pch) - pch;
+           }
 
          // line's too long; add a linefeed if we can
          if (pos>=column && linefeed_here!=NULL)
          {
-            *linefeed_here = '\n';
-            pch = linefeed_here + 1;
+            const char nl[5]="   \n";
+            if( space_len == 1)
+              *linefeed_here = '\n';
+             else
+               memcpy( linefeed_here, 4 - space_len + nl, space_len);
+            pch = linefeed_here + space_len;
             linefeed_here = NULL;
+            space_len = 0;
             pos = 0;
          }
          else
@@ -257,7 +346,7 @@ namespace
 }
 
 std::string
-TextMassager :: fill (const StringView& body) const
+TextMassager :: fill (const StringView& body, bool flowed) const
 {
    std::string retval;
 
@@ -277,7 +366,7 @@ TextMassager :: fill (const StringView& body) const
    typedef std::vector<std::string> strings_t;
    typedef strings_t::const_iterator strings_cit;
    strings_t lines;
-   paragraphs_t paragraphs (get_paragraphs (*this, tmp_body));
+   paragraphs_t paragraphs (get_paragraphs (*this, tmp_body, flowed));
    for (p_it it=paragraphs.begin(), end=paragraphs.end(); it!=end; ++it)
       fill_paragraph (*this, *it, lines);
 
@@ -309,7 +398,8 @@ TextMassager :: mute_quotes (const StringView& text) const
    const char * mute_str = _("> [quoted text muted]");
 
    StringView mytext (text);
-   StringView line;	
+   StringView line;
+
    bool last_line_was_quote = false;
    while (mytext.pop_token (line, '\n'))
    {
@@ -335,7 +425,6 @@ TextMassager :: mute_quotes (const StringView& text) const
    return retval;
 }
 
-
 char*
 TextMassager :: rot13_inplace (char * text)
 {
@@ -358,3 +447,73 @@ TextMassager :: rot13_inplace (char * text)
 
    return text;
 }
+
+std::string
+pan :: subject_to_path (const char * subjectline, const std::string &seperator)
+{
+  gchar *str1, *str2;
+  const char *sep;
+  std::string val (subjectline);
+  //std::string::size_type pos;
+  //stupid hack to silence the compiler
+  GRegexCompileFlags cf0((GRegexCompileFlags)0);
+  GRegexMatchFlags mf0((GRegexMatchFlags)0);
+
+  if (seperator.length() != 1)
+    sep = "-";
+  else switch (seperator[0]) {
+    case ' ':
+    case '-':
+    case '_': sep = seperator.c_str(); break;
+    default : sep = "-"; break;
+  }
+
+  // strip out newspost/Xnews-style multi-part strings
+  GRegex *mp1 =g_regex_new("\\s*(?:[Ff]ile|[Pp]ost)\\s[0-9]+\\s*(?:of|_)\\s*[0-9]+[:\\s]?", cf0, mf0, NULL);
+  str1 = g_regex_replace_literal(mp1, val.c_str(), -1, 0, " ", mf0, NULL);
+  g_regex_unref(mp1);
+
+  // and the rest.  the last check is for pans collapsed part count
+  GRegex *mp2 =g_regex_new("\\s*([\[(]?'?[0-9]+'?\\s*(?:of|/)\\s*'?[0-9]+'?.)|\\(/[0-9]+\\)", cf0, mf0, NULL);
+  str2 = g_regex_replace_literal(mp2, str1, -1, 0, "", mf0, NULL);
+  g_free(str1);
+  g_regex_unref(mp2);
+
+  // try to strip out the filename (may fail if it contains spaces)
+  GRegex *fn =g_regex_new("\"[^\"]+?\" yEnc.*"    "|"
+                          "\\S++\\s++yEnc.*"      "|"
+                          "\"[^\"]+?\\.\\w{2,}\"" "|"
+                          "\\S+\\.\\w{2,4}", cf0, mf0, NULL);
+  str1 = g_regex_replace_literal(fn, str2, -1, 0, "", mf0, NULL);
+  g_free(str2);
+  g_regex_unref(fn);
+
+  // try to strip out any byte counts
+  GRegex *cnt =g_regex_new("\\[?[0-9]+\\s*(?:[Bb](ytes)?|[Kk][Bb]?)\\]?", cf0, mf0, NULL);
+  str2 = g_regex_replace_literal(cnt, str1, -1, 0, "", mf0, NULL);
+  g_free(str1);
+  g_regex_unref(cnt);
+
+  // remove any illegal / annoying characters
+  GRegex *badc =g_regex_new("[\\\\/<>|*?'\"\\.\\s]+", cf0, mf0, NULL);
+  str1 = g_regex_replace_literal(badc, str2, -1, 0, sep, mf0, NULL);
+  g_free(str2);
+  g_regex_unref(badc);
+
+  // remove any extraneous whitespace, '_', & '-'
+  GRegex *ext =g_regex_new("[\\s_-]{2,}", cf0, mf0, NULL);
+  str2 = g_regex_replace_literal(ext, str1, -1, 0, sep, mf0, NULL);
+  g_free(str1);
+  g_regex_unref(ext);
+
+  // remove leading & trailing junk
+  ext =g_regex_new("(^[\\s_-]+)|([\\s_-]+$)", cf0, mf0, NULL);
+  str1 = g_regex_replace_literal(ext, str2, -1, 0, "", mf0, NULL);
+  g_free(str2);
+  g_regex_unref(ext);
+
+  val=str1;
+  g_free(str1);
+  //std::cout << "\nSubject was: '" << subjectline << "'\nSubject now: '" << val << "'" << std::endl;
+  return val;
+}
